/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* MODULE NAME..... QCE1                                             */
/* DESCRIPTION..... This module serves as an entry point for the     */
/*                  Shared Object Linkage Driver (SOLD) test cases.  */
/*                                                                   */
/* FUNCTION... QCE1 is a function called by test cases               */
/*                     1, 4, 51, 52, 56, 62, 63                      */
/*             of the SOLD driver. The test case number is passed to */
/*             this function via the ECB workarea field EBW000.      */
/*                                                                   */
/*             qce1a is an internal function, called only by QCE1.   */
/*             It takes a function pointer as a parameter.  The      */
/*             function first checks the static data member's value  */
/*             for correctness and then calls the function passed.   */
/*                                                                   */
/*             check_static is a simple function to check the value  */
/*             of the static data member from outside this module.   */
/*             It returns the value of the data member.              */
/*                                                                   */
/* RESTRICTIONS....                                                  */
/*                                                                   */
/* INPUT.....                                                        */
/*                                                                   */
/* NORMAL RETURN...                                                  */
/*                                                                   */
/* ERROR RETURN....                                                  */
/*                                                                   */
/* EXTERNAL ROUTINES.....                                            */
/*                                                                   */
/*********************************************************************/
#include <c_sold.h>
#include <stdio.h>
#include <tpf/tpfapi.h>

void QCE2(void);                                                                //C64JK
void QSB8(struct TPF_regs *);                                                   //C64JK
void QSB6(struct TPF_regs *);                                                   //C64JK
void QSB5( struct TPF_regs * );

static int stint = 0;                                                           //C64JK

void qce1a(int (*func)(int *))
{
  int * ptr = &stint;                                                           //C64JK

  // Change value of static variable and verify contents
  stint++;                                                                      //C64JK
  if( stint != 52 ) {                                                           //C64JK
    bad_static_msg("QCE1", test_case);                                          //C64JK
  }
  // Call func and check return value
  if( func(ptr) != 52 ) {                                                       //C64JK
    bad_return_msg("QCE1", "QSC1", test_case);                                  //C64JK
  }
}

int check_static()
{
  return stint;                                                                 //C64JK
}

/* Note: The parameters are only valid for test case 12 */
/*       Everyone else calls this as void QCE1(void)    */
long QCE1(char mychar, double mydouble, long mylong, short myshort, void *myptr, int myint)
{
  // Change static data member value
  stint = 51;                                                                   //C64JK
  struct TPF_regs regs;
  struct pat *pat_addr;                                                         //d10433f_1
  char prog_name[PAT_PROG_NAME_LEN+PAT_VERS_CODE_LEN+1] = {0};                  //d10875

  if (test_mode == SHOW_VER)  {                                                 //f5555
     pat_addr = (void*) tpf_ecbp3()->ce3patg;                                   //f12472
     memcpy(prog_name, pat_addr->patname, PAT_PROG_NAME_LEN+PAT_VERS_CODE_LEN); //d10875
     show_prog_version(prog_name, pat_addr->patacnum);                          //f12305
  }

  switch(test_case) {
  case 1:
    QCE2();                                                                     //C64JK
    break;
  case 4:
    qce1a(qsc1a);                                                               //C64JK
    break;
  case 12:
       if ((mychar != 'F') ||                                                   //C64CF
           (mydouble != 3.141526)  ||                                           //C64CF
           (mylong != 42000000000) ||                                           //C64CF
           (myshort != -12345)     ||                                           //C64CF
           (myint   != 1234567890) ||                                           //C64CF
           (strcmp(myptr, "QCE1") != 0)) {                                      //C64CF
              bad_param_msg("QCE1", test_case);                                 //C64CF
       }                                                                        //C64CF
                                                                                //C64CF
       return 42;                                                               //C64CF
  case 51:
  case 56:
    /* Set up registers */
    regs.r0 = 568945021;                                                        //C64JK
    regs.r1 = 1051;                                                             //C64JK
    regs.r2 = 59865;                                                            //C64JK
    regs.r3 = 897541;                                                           //C64JK
    regs.r4 = 23;                                                               //C64JK
    regs.r5 = 567986541;                                                        //C64JK
    regs.r6 = (long)&regs.r0;                                                   //C64JK
    memcpy(&regs.r7, "\0\0\0\0SOLD", sizeof(regs.r7));                          //C64JK
    QSB8(TPF_regs_Call64(&regs));                                               //C64JK
    /* Verify results */
    if((char)regs.r7 != 'P')                                                    //C64JK
       bad_return_msg("QCE1", "QSB8", test_case);                               //C64JK
    break;
  case 52:
    /* Set up registers */
    regs.r0 = 2003;                                                             //C64JK
    regs.r1 = -1000;                                                            //C64JK
    regs.r2 = 600000;                                                           //C64JK
    regs.r3 = -503599;                                                          //C64JK
    regs.r4 = -96000;                                                           //C64JK
    regs.r5 = 2602;                                                             //C64JK
    regs.r6 = (long)&regs.r0;                                                   //C64JK
    QSB6(TPF_regs_Call64(&regs));                                               //C64JK
    /* Check results */
    if( (char)regs.r7 != 'P' )                                                  //C64JK
       bad_return_msg("QCE1", "QSB6", test_case);                               //C64JK

    break;
   case 62:
   case 63:

     /* Add a message to the trace table to be examined in the dump */
     tpf_trace_info( "Trace Message from QCE1 (TRG_3)" );
     QSB5( NULL );

     break;
  default:
    bad_case_msg("QCE1", test_case);                                            //C64JK
  }

  return -1;
}
