/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* MODULE NAME..... QCE2                                             */
/* DESCRIPTION..... This module serves as an entry point for the     */
/*                  Shared Object Linkage Driver (SOLD) test cases.  */
/*                                                                   */
/* FUNCTION... QCE2 is a function called by test cases               */
/*                     1, 2, 6, 8, 51, 53, 54, 63                    */
/*             of the SOLD driver. The test case number is passed to */
/*             this function via the ECB workarea field EBW000.      */
/*                                                                   */
/*             qce2a is an internal function, called only by QCE2.   */
/*             It takes a string as a parameter.  The string must    */
/*             be 8 characters long and the function checks this.    */
/*             qce2a then calls the next function for the given test */
/*             case.                                                 */
/*                                                                   */
/* RESTRICTIONS....                                                  */
/*                                                                   */
/* INPUT.....                                                        */
/*                                                                   */
/* NORMAL RETURN...                                                  */
/*                                                                   */
/* ERROR RETURN....                                                  */
/*                                                                   */
/* EXTERNAL ROUTINES.....                                            */
/*                                                                   */
/*********************************************************************/
#include <c_sold.h>
#include <tpf/tpfapi.h>
#include <stdio.h>
#include <string.h>

void QSB6(struct TPF_regs *);                                                    //C64JK
void QSBC(struct TPF_regs *);                                                    //C64JK
void test_prlgc_31();
void test_prlgc_64();


	
void qce2a( char * str )
{
  int sum = 210;                                                                //C64JK

  // Parameter Checking
  if( strlen(str) != 8 ) {                                                      //C64JK
    bad_param_msg("QCE2", test_case);                                           //C64JK
  }

  switch(test_case) {
  case 1:
    // Parameters add up to sum
    if( !qsc3b(10, 20, 30, 40, 50, 60, &sum ) ) {                               //C64JK
      bad_return_msg("QCE2", "QSC3", test_case);                                //C64JK
    }
    break;
  case 6:
    sum = 945;                                                                  //C64JK
    if( !qsc3b(780, 23456, -40000, 12950, 5500, -1741, &sum) ) {                //C64JK
      bad_return_msg("QCE2", "QSC3", test_case);                                //C64JK
    }
    break;
  default:
    bad_case_msg("QCE2", test_case);                                            //C64JK
  }
}

void QCE2()
{
  struct TPF_regs regs;                                                         //C64JK

  struct pat *pat_addr;                                                         //d10433f_1
  char prog_name[PAT_PROG_NAME_LEN+PAT_VERS_CODE_LEN+1] = {0};                  //d10875

  if (test_mode == SHOW_VER)  {                                                 //f5555
     pat_addr = (void*) tpf_ecbp3()->ce3patg;                                   //f12472
     memcpy(prog_name, pat_addr->patname, PAT_PROG_NAME_LEN+PAT_VERS_CODE_LEN); //d10875
     show_prog_version(prog_name, pat_addr->patacnum);                          //f12305
  }

  switch(test_case) {
  case 1:
    // Check value of static variable from QCE1
    if( check_static() != 51 ) {                                                //C64JK
      bad_static_msg("QCE2", test_case);                                        //C64JK
    }
    qce2a("internal");                                                          //C64JK
    break;
  case 2:
    // Parameters must add up to 2003
    if( qsc2a(-1000, 600000, -503599, -96000, 1500, 1102, qsc2b) != 2003 ) {    //C64JK
      bad_return_msg("QCE2", "QSC2", test_case);                                //C64JK
    }
    break;
  case 6:                                                                       //C64JK
    qce2a("linkages");                                                          //C64JK
    break;                                                                      //C64JK
  case 8:                                                                       //C64JK
    // Nothing left to do. Last call along the line.
    break;                                                                      //C64JK

  case 51:
     /* Set up registers */
     regs.r0 = 579875;                                                          //C64JK
     regs.r1 = 89754;                                                           //C64JK
     regs.r2 = -6573328;                                                        //C64JK
     regs.r3 = 567352;                                                          //C64JK
     regs.r4 = -329855;                                                         //C64JK
     regs.r5 = 29580;                                                           //C64JK
     regs.r6 = 5636622;                                                         //C64JK
     QSBC(&regs);                                                               //C64JK
     /* Check results */
     if( (char)regs.r7 != 'P' )                                                 //C64JK
        bad_return_msg("QCE2", "QSBC", test_case);                              //C64JK
     break;
  case 53:
  case 54:
    /* Set up registers */
    regs.r0 = 2003;                                                             //C64JK
    regs.r1 = -1000;                                                            //C64JK
    regs.r2 = 600000;                                                           //C64JK
    regs.r3 = -503599;                                                          //C64JK
    regs.r4 = -96000;                                                           //C64JK
    regs.r5 = 2602;                                                             //C64JK
    regs.r6 = (long)&regs.r0;                                                   //C64JK
    QSB6(TPF_regs_Call64(&regs));                                               //C64JK
    /* Check results */
    if( regs.r7 != regs.r0 )                                                    //C64JK
       bad_return_msg("QCE2", "QSB6", test_case);                               //C64JK
    break;
  case 61:
    test_prlgc_31();
    test_prlgc_64();
    break;
  case 63:

     /* Add a message to the trace table to be examined in the dump */
     tpf_trace_info( "Trace Message from QCE2 (TRG_2)" );

     break;
  default:
    bad_case_msg("QCE2", test_case);                                            //C64JK
  }
}
