/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* MODULE NAME..... QCE3                                             */
/* DESCRIPTION..... This module serves as an entry point for the     */
/*                  Shared Object Linkage Driver (SOLD) test cases.  */
/*                                                                   */
/* FUNCTION... QCE3 is a function called by test cases               */
/*                     3, 7, 52, 53, 54, 55, 57, 60, 62, 63          */
/*             of the SOLD driver. The test case number is passed to */
/*             this function via the ECB workarea field EBW000.      */
/*                                                                   */
/* RESTRICTIONS....                                                  */
/*                                                                   */
/* INPUT.....                                                        */
/*                                                                   */
/* NORMAL RETURN...                                                  */
/*                                                                   */
/* ERROR RETURN....                                                  */
/*                                                                   */
/* EXTERNAL ROUTINES.....                                            */
/*                                                                   */
/*********************************************************************/
#include <c_sold.h>
#include <tpf/tpfregs.h>
#include <tpf/tpfapi.h>

void QSBD(struct TPF_regs *);                                                   //C64JK
void QSBF(struct TPF_regs *);                                                   //C64JK
void QSB1( struct TPF_regs * );

void QCE3()
{
  struct TPF_regs regs;                                                         //C64JK
  int sum = -7102;                                                              //C64JK

  struct pat *pat_addr;                                                         //d10433f_1
  char prog_name[PAT_PROG_NAME_LEN+PAT_VERS_CODE_LEN+1] = {0};                  //d10875

  if (test_mode == SHOW_VER)  {                                                 //f5555
     pat_addr = (void*) tpf_ecbp3()->ce3patg;                                   //f12472
     memcpy(prog_name, pat_addr->patname, PAT_PROG_NAME_LEN+PAT_VERS_CODE_LEN); //d10875
     show_prog_version(prog_name, pat_addr->patacnum);                          //f12305
  }

  switch(test_case) {
  case 3:
    if( qsc1b('a', 'b', 'c', 'd', 'e', 'f', qsc3a) != 'P' ) {                   //C64JK
      bad_return_msg("QCE3", "QSC1", test_case);                                //C64JK
    }
    break;
  case 7:                                                                       //C64JK
    if( !qsc3b(11255, -6755, 12005, -34002, 19557, -9162, &sum) ) {             //C64JK
      bad_return_msg("QCE3", "QSC3", test_case);                                //C64JK
    }                                                                           //C64JK
    break;                                                                      //C64JK
  case 52:
  case 55:
  case 57:

     /* Set up TPF_regs */
     regs.r0 = -11602;                                                          //C64JK
     regs.r4 = 41486;                                                           //C64JK
     regs.r5 = -93082;                                                          //C64JK
     regs.r6 = 39994;                                                           //C64JK
     QSBF(&regs);                                                               //C64JK
     /* Check result */
     if( (char)regs.r7 != 'P' ) {                                               //C64JK
       bad_return_msg("QCE3", "QSBF", test_case);                               //C64JK
     }
     break;
  case 53:
  case 54:
     /* Set up TPF_regs */
     regs.r0 = -741750;                                                         //C64JK
     regs.r1 = 457892;                                                          //C64JK
     regs.r2 = -898524;                                                         //C64JK
     regs.r3 = -324578;                                                         //C64JK
     regs.r4 = 23459;                                                           //C64JK
     regs.r5 = 1;                                                               //C64JK
     regs.r6 = (long)&regs.r0;                                                  //C64JK
     memcpy(&regs.r7, "\0\0\0\0SOLD", sizeof(regs.r7));                         //C64JK
     QSBD(&regs);                                                               //C64JK
     break;
  case 12:                                                                      //C64CF
  case 60:
     /* End of the line */
     break;

  case 62:
  case 63:

     /* Add a message to the trace table to be examined in the dump */
     tpf_trace_info( "Trace Message from QCE3 (TRG_1)" );
     QSB1( NULL );

     break;
  default:
     bad_case_msg("QCE3", test_case);                                           //C64JK
  }

  return;
}
