/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* JKOSKI      1/28/2005                     Added test cases 101 -  */
/*                                           123 for function trace  */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* MODULE NAME..... QME1                                             */
/* DESCRIPTION..... This module serves as an entry point using the   */
/*                  main() function for the Shared Object Linkage    */
/*                  Driver (SOLD) test cases.                        */
/*                                                                   */
/* FUNCTION... QME1 is a function called by test cases               */
/*                     5, 8, 55, 101 - 123                           */
/*             of the SOLD driver. The test case number is passed to */
/*             this function through the parameter array argv and    */
/*             placed into the ECB workarea field EBW000.            */
/*                                                                   */
/*             qme1a is a function that requires four strings to be  */
/*             passed in.  When concatenated these strings should    */
/*             produce "Shared Object Linkage Driver" If the         */
/*             arguments produce this string 1 is returned, 0        */
/*             otherwise.                                            */
/*                                                                   */
/*             qme1b is a function that takes four strings and one   */
/*             function pointer as arguments.  The function pointer  */
/*             is called with the four strings passed to it.  This   */
/*             function returns 'P' if successful, 'F' otherwise.    */
/*                                                                   */
/* RESTRICTIONS....                                                  */
/*                                                                   */
/* INPUT.....                                                        */
/*                                                                   */
/* NORMAL RETURN...                                                  */
/*                                                                   */
/* ERROR RETURN....                                                  */
/*                                                                   */
/* EXTERNAL ROUTINES.....                                            */
/*                                                                   */
/*********************************************************************/
#include <tpf/tpfapi.h>
#include <stdio.h>
#include <stdlib.h>
#include <c_sold.h>

void QSB7(struct TPF_regs *);                                                   //C64JK

int qme1a(char *str1, char *str2, char *str3, char *str4)
{
   char * bigString = malloc(BUFNO);                                            //C64JK
   sprintf(bigString, "%s %s %s %s", str1, str2, str3, str4);                   //C64JK
   if( strcmp(bigString,"Shared Object Linkage Driver") ) {                     //C64JK
      bad_param_msg("QME1", test_case);                                         //C64JK
   }
   free(bigString);                                                             //C64CF

   return 1;                                                                    //C64JK
}

char qme1b(char *str1, char *str2, char *str3, char *str4,
           int (*func)(char *, char *, char *, char *))
{
   if( !func(str1, str2, str3, str4) ) {                                        //C64JK
      bad_return_msg("QME1", "QME1", test_case);                                //C64JK
   }

   return 'P';                                                                  //C64JK
}

int main(int argc, char **argv)
{
   test_case = atoi(argv[1]);                                                   //C64JK
   test_mode = atoi(argv[2]);                                                   //d10433f_1
   int zero = 0;		/* to avoid compiler warning */
   struct TPF_regs regs;                                                        //C64JK
   struct pat *pat_addr;                                                        //d10433f_1
   char prog_name[PAT_PROG_NAME_LEN+PAT_VERS_CODE_LEN+1] = {0};                 //d10875

  if (test_mode == SHOW_VER)  {                                                 //f5555
     pat_addr = (void*) tpf_ecbp3()->ce3patg;                                   //f12472
     memcpy(prog_name, pat_addr->patname, PAT_PROG_NAME_LEN+PAT_VERS_CODE_LEN); //d10875
     show_prog_version(prog_name, pat_addr->patacnum);                          //f12305
  }

   switch(test_case) {
   case 5:
       if( qme1b("Shared", "Object", "Linkage", "Driver", qme1a) != 'P' ) {     //C64JK
          bad_return_msg("QME1", "QME1", test_case);                            //C64JK
       }
       break;
   case 8:
       if( qsc3a('6', '4', 'B', 'i', 't', 'C', "64BitC") != 'P' ) {             //C64JK
          bad_return_msg("QME1", "QSC3", test_case);                            //C64JK
       }
       break;
   case 55:
       /* Set up registers */
       regs.r0 = 2003;                                                          //C64JK
       regs.r1 = 1881;                                                          //C64JK
       regs.r2 = 101880;                                                        //C64JK
       regs.r3 = -41486;                                                        //C64JK
       regs.r4 = -75000;                                                        //C64JK
       regs.r5 = 10000;                                                         //C64JK
       regs.r6 = 4728;                                                          //C64JK
       QSB7(TPF_regs_Call64(&regs));                                            //C64JK
       /* Check results */
       if( (char)regs.r7 != 'P' ) {                                             //C64JK
          bad_return_msg("QME1", "QSB7", test_case);                            //C64JK
       }
       break;
   case 101:
      /*-------------------------------------------------------------------*/
      /* Test char parameters and char return parameter                    */
      /*-------------------------------------------------------------------*/
      ;
      char char1 = 'A';                                                         //f11086
      char char2 = 'b';                                                         //f11086
      char char3 = 'C';                                                         //f11086
      char char4 = 'd';                                                         //f11086
      char char5 = 'E';                                                         //f11086
      char char6 = 'f';                                                         //f11086
      char charreturn = ' ';                                                    //f11086

      charreturn = testchar1(char1);                                            //f11086
      charreturn = testchar2(char1,char2);                                      //f11086
      charreturn = testchar3(char1,char2,char3);                                //f11086
      charreturn = testchar4(char1,char2,char3,char4);                          //f11086
      charreturn = testchar5(char1,char2,char3,char4,char5);                    //f11086
      charreturn = testchar6(char1,char2,char3,char4,char5,char6);              //f11086
      break;
   case 102:
      /*-------------------------------------------------------------------*/
      /* Test unsigned char parameters and unsigned char return parameter  */
      /*-------------------------------------------------------------------*/
      ;
      unsigned char uchar1 = 'A';                                               //f11086
      unsigned char uchar2 = 'b';                                               //f11086
      unsigned char uchar3 = 'C';                                               //f11086
      unsigned char uchar4 = 'd';                                               //f11086
      unsigned char uchar5 = 'E';                                               //f11086
      unsigned char uchar6 = 'f';                                               //f11086
      unsigned char ucharreturn = ' ';                                          //f11086

      ucharreturn = testuchar1(uchar1);                                         //f11086
      ucharreturn = testuchar2(uchar1,uchar2);                                  //f11086
      ucharreturn = testuchar3(uchar1,uchar2,uchar3);                           //f11086
      ucharreturn = testuchar4(uchar1,uchar2,uchar3,uchar4);                    //f11086
      ucharreturn = testuchar5(uchar1,uchar2,uchar3,uchar4,uchar5);             //f11086
      ucharreturn = testuchar6(uchar1,uchar2,uchar3,uchar4,uchar5,uchar6);      //f11086
      break;
   case 103:
      /*-------------------------------------------------------------------*/
      /* Test short parameters and short return parameter                  */
      /*-------------------------------------------------------------------*/
      ;
      short short1 = 1;                                                         //f11086
      short short2 = 2;                                                         //f11086
      short short3 = 3;                                                         //f11086
      short short4 = 4;                                                         //f11086
      short short5 = 5;                                                         //f11086
      short short6 = 6;                                                         //f11086
      short shortreturn = 0;                                                    //f11086

      shortreturn = testshort1(short1);                                         //f11086
      shortreturn = testshort2(short1,short2);                                  //f11086
      shortreturn = testshort3(short1,short2,short3);                           //f11086
      shortreturn = testshort4(short1,short2,short3,short4);                    //f11086
      shortreturn = testshort5(short1,short2,short3,short4,short5);             //f11086
      shortreturn = testshort6(short1,short2,short3,short4,short5,short6);      //f11086
      break;
   case 104:
      /*-------------------------------------------------------------------*/
      /* Test unsigned short parameters and unsigned short return parameter*/
      /*-------------------------------------------------------------------*/
      ;
      unsigned short ushort1 = 1;                                               //f11086
      unsigned short ushort2 = 2;                                               //f11086
      unsigned short ushort3 = 3;                                               //f11086
      unsigned short ushort4 = 4;                                               //f11086
      unsigned short ushort5 = 5;                                               //f11086
      unsigned short ushort6 = 6;                                               //f11086
      unsigned short ushortreturn = 0;                                          //f11086

      ushortreturn = testushort1(ushort1);                                      //f11086
      ushortreturn = testushort2(ushort1,ushort2);                              //f11086
      ushortreturn = testushort3(ushort1,ushort2,ushort3);                      //f11086
      ushortreturn = testushort4(ushort1,ushort2,ushort3,ushort4);              //f11086
      ushortreturn = testushort5(ushort1,ushort2,ushort3,ushort4,ushort5);      //f11086
      ushortreturn = testushort6(ushort1,ushort2,ushort3,ushort4,ushort5,ushort6);  //f11086
      break;
   case 105:
      /*-------------------------------------------------------------------*/
      /* Test int parameters and int return parameter                      */
      /*-------------------------------------------------------------------*/
      ;
      int int1 = 1;                                                             //f11086
      int int2 = 2;                                                             //f11086
      int int3 = 3;                                                             //f11086
      int int4 = 4;                                                             //f11086
      int int5 = 5;                                                             //f11086
      int int6 = 6;                                                             //f11086
      int intreturn = 0;                                                        //f11086

      intreturn = testint1(int1);                                               //f11086
      intreturn = testint2(int1,int2);                                          //f11086
      intreturn = testint3(int1,int2,int3);                                     //f11086
      intreturn = testint4(int1,int2,int3,int4);                                //f11086
      intreturn = testint5(int1,int2,int3,int4,int5);                           //f11086
      intreturn = testint6(int1,int2,int3,int4,int5,int6);                      //f11086
      break;
   case 106:
      /*-------------------------------------------------------------------*/
      /* Test unsigned int parameters and unsigned int return parameter    */
      /*-------------------------------------------------------------------*/
      ;
      unsigned int uint1 = 1;                                                   //f11086
      unsigned int uint2 = 2;                                                   //f11086
      unsigned int uint3 = 3;                                                   //f11086
      unsigned int uint4 = 4;                                                   //f11086
      unsigned int uint5 = 5;                                                   //f11086
      unsigned int uint6 = 6;                                                   //f11086
      unsigned int uintreturn = 0;                                              //f11086

      uintreturn = testuint1(uint1);                                            //f11086
      uintreturn = testuint2(uint1,uint2);                                      //f11086
      uintreturn = testuint3(uint1,uint2,uint3);                                //f11086
      uintreturn = testuint4(uint1,uint2,uint3,uint4);                          //f11086
      uintreturn = testuint5(uint1,uint2,uint3,uint4,uint5);                    //f11086
      uintreturn = testuint6(uint1,uint2,uint3,uint4,uint5,uint6);              //f11086
      break;
   case 107:
      /*-------------------------------------------------------------------*/
      /* Test long parameters and long return parameter                    */
      /*-------------------------------------------------------------------*/
      ;
      long long1 = 1;                                                           //f11086
      long long2 = 2;                                                           //f11086
      long long3 = 3;                                                           //f11086
      long long4 = 4;                                                           //f11086
      long long5 = 5;                                                           //f11086
      long long6 = 6;                                                           //f11086
      long longreturn = 0;                                                      //f11086

      longreturn = testlong1(long1);                                            //f11086
      longreturn = testlong2(long1,long2);                                      //f11086
      longreturn = testlong3(long1,long2,long3);                                //f11086
      longreturn = testlong5(long1,long2,long3,long4,long5);                    //f11086
      longreturn = testlong6(long1,long2,long3,long4,long5,long6);              //f11086
      break;
   case 108:
      /*-------------------------------------------------------------------*/
      /* Test unsigned long parameters and unsigned long return parameter  */
      /*-------------------------------------------------------------------*/
      ;
      unsigned long ulong1 = 1;                                                 //f11086
      unsigned long ulong2 = 2;                                                 //f11086
      unsigned long ulong3 = 3;                                                 //f11086
      unsigned long ulong4 = 4;                                                 //f11086
      unsigned long ulong5 = 5;                                                 //f11086
      unsigned long ulong6 = 6;                                                 //f11086
      unsigned long ulongreturn = 0;                                            //f11086

      ulongreturn = testulong1(ulong1);                                         //f11086
      ulongreturn = testulong2(ulong1,ulong2);                                  //f11086
      ulongreturn = testulong3(ulong1,ulong2,ulong3);                           //f11086
      ulongreturn = testulong4(ulong1,ulong2,ulong3,ulong4);                    //f11086
      ulongreturn = testulong5(ulong1,ulong2,ulong3,ulong4,ulong5);             //f11086
      ulongreturn = testulong6(ulong1,ulong2,ulong3,ulong4,ulong5,ulong6);      //f11086
      break;
   case 109:
      /*-------------------------------------------------------------------*/
      /* Test case 8                                                       */
      /* Test long long parameters and long long return parameter          */
      /*-------------------------------------------------------------------*/
      ;
      long long longlong1 = 1;                                                  //f11086
      long long longlong2 = 2;                                                  //f11086
      long long longlong3 = 3;                                                  //f11086
      long long longlong4 = 4;                                                  //f11086
      long long longlong5 = 5;                                                  //f11086
      long long longlong6 = 6;                                                  //f11086
      long long longlongreturn = 0;                                             //f11086

      longlongreturn = testlonglong1(longlong1);                                //f11086
      longlongreturn = testlonglong2(longlong1,longlong2);                      //f11086
      longlongreturn = testlonglong3(longlong1,longlong2,longlong3);            //f11086
      longlongreturn = testlonglong4(longlong1,longlong2,longlong3,longlong4);  //f11086
      longlongreturn = testlonglong5(longlong1,longlong2,
                                    longlong3,longlong4,longlong5);             //f11086
      longlongreturn = testlonglong6(longlong1,longlong2,longlong3,
                                    longlong4,longlong5,longlong6);             //f11086
      break;
   case 110:
      /*-------------------------------------------------------------------*/
      /* Test unsigned long long parameters and                            */
      /* unsigned long long return parameter                               */
      /*-------------------------------------------------------------------*/
      ;
      unsigned long long ulonglong1 = 1;                                        //f11086
      unsigned long long ulonglong2 = 2;                                        //f11086
      unsigned long long ulonglong3 = 3;                                        //f11086
      unsigned long long ulonglong4 = 4;                                        //f11086
      unsigned long long ulonglong5 = 5;                                        //f11086
      unsigned long long ulonglong6 = 6;                                        //f11086
      unsigned long long ulonglongreturn = 0;                                   //f11086

      ulonglongreturn = testulonglong1(ulonglong1);                             //f11086
      ulonglongreturn = testulonglong2(ulonglong1,ulonglong2);                  //f11086
      ulonglongreturn = testulonglong3(ulonglong1,ulonglong2,ulonglong3);       //f11086
      ulonglongreturn = testulonglong4(ulonglong1,ulonglong2,
                                       ulonglong3,ulonglong4);                  //f11086
      ulonglongreturn = testulonglong5(ulonglong1,ulonglong2,
                                       ulonglong3,ulonglong4,ulonglong5);       //f11086
      ulonglongreturn = testulonglong6(ulonglong1,ulonglong2,ulonglong3,
                                       ulonglong4,ulonglong5,ulonglong6);       //f11086
      break;
   case 111:
      /*-------------------------------------------------------------------*/
      /* Test   float parameters and   float return parameter              */
      /*-------------------------------------------------------------------*/
      ;
      float  float1 = 1.0;                                                      //f11086
      float  float2 = 2.0;                                                      //f11086
      float  float3 = 3.0;                                                      //f11086
      float  float4 = 4.0;                                                      //f11086
      float  float5 = 5.0;                                                      //f11086
      float  float6 = 6.0;                                                      //f11086
      float  floatreturn = 0.0;                                                 //f11086

      floatreturn = testfloat1(float1);                                         //f11086
      floatreturn = testfloat2(float1, float2);                                 //f11086
      floatreturn = testfloat3(float1,float2,float3);                           //f11086
      floatreturn = testfloat4(float1,float2,float3,float4);                    //f11086
      floatreturn = testfloat5(float1,float2,float3,float4,float5);             //f11086
      floatreturn = testfloat6(float1,float2,float3,float4,float5,float6);      //f11086
      break;
   case 112:
      /*-------------------------------------------------------------------*/
      /* Test   double parameters and   double return parameter            */
      /*-------------------------------------------------------------------*/
      ;
      double  double1 = 1.0;                                                    //f11086
      double  double2 = 2.0;                                                    //f11086
      double  double3 = 3.0;                                                    //f11086
      double  double4 = 4.0;                                                    //f11086
      double  double5 = 5.0;                                                    //f11086
      double  double6 = 6.0;                                                    //f11086
      double  doublereturn = 0.0;                                               //f11086

      doublereturn = testdouble1(double1);                                      //f11086
      doublereturn = testdouble2(double1, double2);                             //f11086
      doublereturn = testdouble3(double1,double2,double3);                      //f11086
      doublereturn = testdouble4(double1,double2,double3,double4);              //f11086
      doublereturn = testdouble5(double1,double2,double3,double4,double5);      //f11086
      doublereturn = testdouble6(double1,double2,double3,double4,double5,
                                 double6);                                      //f11086
      break;
   case 113:
      /*-------------------------------------------------------------------*/
      /* Test long double parameters and long double return parameter      */
      /*-------------------------------------------------------------------*/
      ;
      long double ld1 = 1.0;                                                    //f11086
      long double ld2 = 2.0;                                                    //f11086
      long double ld3 = 3.0;                                                    //f11086
      long double ld4 = 4.0;                                                    //f11086
      long double ld5 = 5.0;                                                    //f11086
      long double ld6 = 6.0;                                                    //f11086
      long double ldreturn = 0.0;                                               //f11086

      ldreturn = testlongdouble1(ld1);                                          //f11086
      ldreturn = testlongdouble2(ld1,ld2);                                      //f11086
      ldreturn = testlongdouble3(ld1,ld2,ld3);                                  //f11086
      ldreturn = testlongdouble4(ld1,ld2,ld3,ld4);                              //f11086
      ldreturn = testlongdouble5(ld1,ld2,ld3,ld4,ld5);                          //f11086
      ldreturn = testlongdouble6(ld1,ld2,ld3,ld4,ld5,ld6);                      //f11086
      break;
   case 114:
      /*-------------------------------------------------------------------*/
      /* Test variable parameter                                           */
      /*-------------------------------------------------------------------*/
      ;
      char varchar3 = 'A';                                                      //f11086
      int varint3 = 3;                                                          //f11086
      int varret3 = 0;                                                          //f11086
      varret3 = testvar3(varchar3, varint3);                                    //f11086
      break;
   case 115:
      /*-------------------------------------------------------------------*/
      /* Test void parameters and void return parameter                    */
      /*-------------------------------------------------------------------*/
      free(0);       /* invalid free, but just used to test function trace */   //f11086
      testvoid1();                                                              //f11086
      break;
   case 116:
      /*-------------------------------------------------------------------*/
      /* Test tpf_trace_info() function example in TPF documentation.      */
      /* Return code = 35 when trace is active (ZSTRC ALTER TRACE)         */
      /* Return code =  0 when trace is not active (ZSTRC ALTER NOTRACE)   */
      /*-------------------------------------------------------------------*/
      ;
      int doc_rc = 1555;               /* init to garbage return code   */      //f11086
      char * fishy_msg;                                                         //f11086

      fishy_msg = "So long and thanks for all the fish";                        //f11086
      doc_rc = tpf_trace_info(fishy_msg);                                       //f11086
      if(test_mode != DUMP_YES)
         printf("tpf_trace_info test of doc msg. Return code=%d\n", doc_rc);    //f11086
      break;
   case 117:
      /*-------------------------------------------------------------------*/
      /* Test tpf_trace_info() function with 80-character message          */
      /* (excluding ending '\0' null character)                            */
      /* Return code = 80 when trace is active (ZSTRC ALTER TRACE)         */
      /* Return code =  0 when trace is not active (ZSTRC ALTER NOTRACE)   */
      /*-------------------------------------------------------------------*/
      ;
      int test80_rc = 1666;            /* init to garbage return code   */      //f11086
      char * msg_80;                                                            //f11086

      msg_80 =
   "Test tpf_trace_info function with 80-character message...should not be truncated";
      test80_rc = tpf_trace_info(msg_80);                                       //f11086
      if(test_mode != DUMP_YES)
         printf("tpf_trace_info test of 80-char msg. Return code=%d\n", test80_rc);//f11086
      break;
   case 118:
      /*-------------------------------------------------------------------*/
      /* Test tpf_trace_info() function with 81-character message          */
      /* (excluding ending '\0' null character)                            */
      /* Return code = 80 when trace active (ZSTRC ALTER TRACE)            */
      /* Return code =  0 when trace is not active (ZSTRC ALTER NOTRACE)   */
      /*-------------------------------------------------------------------*/
      ;
      int test81_rc = 1777;            /* init to garbage return code   */      //f11086
      char * msg_81;                                                            //f11086

      msg_81 =
   "Test tpf_trace_info function, with 81-character message which should be truncated";
      test81_rc = tpf_trace_info(msg_81);                                       //f11086
      if(test_mode != DUMP_YES)
         printf("tpf_trace info test of 81-char msg. Return code=%d\n", test81_rc);//f11086
      break;
   case 119:
      /*-------------------------------------------------------------------*/
      /* Test tpf_trace_info() function with 90-character message          */
      /* (excluding ending '\0' null character)                            */
      /* Return code = 80 when trace active (ZSTRC ALTER TRACE)            */
      /* Return code =  0 when trace is not active (ZSTRC ALTER NOTRACE)   */
      /*-------------------------------------------------------------------*/
      ;
      int test90_rc = 1888;            /* init to garbage return code   */      //f11086
      char * msg_90;                                                            //f11086

      msg_90 =
   "Test tpf_trace_info function, with 90-character message which should be truncated by code.";
      test90_rc = tpf_trace_info(msg_90);                                       //f11086
      if(test_mode != DUMP_YES)
         printf("tpf_trace info test of 90-char msg. Return code=%d\n", test90_rc);//f11086
      break;
   case 120:
      /*-------------------------------------------------------------------*/
      /* Test tpf_trace_info() with 0-character message                    */
      /* (excluding ending '\0' null character)                            */
      /* Return code =  0 when trace is active (ZSTRC ALTER TRACE)         */
      /* Return code =  0 when trace is not active (ZSTRC ALTER NOTRACE)   */
      /*-------------------------------------------------------------------*/
      ;
      int test00_rc = 1999;            /* init to garbage return code   */      //f11086
      char * msg_00;                                                            //f11086

      msg_00 = "";                  /* contains only the null character */      //f11086
      test00_rc = tpf_trace_info(msg_00);                                       //f11086
      if(test_mode != DUMP_YES)
         printf("tpf_trace_info test of 0-char msg. Return code=%d\n", test00_rc); //f11086
      break;
   case 121:
      /*-------------------------------------------------------------------*/
      /* Test typedef                                                      */
      /*-------------------------------------------------------------------*/
      ;
      FREDINT typedefretint = 0;                                                //f11086
      FREDINT typedefint = 3;                                                   //f11086
      FREDCHAR typedefchar = 0xC1;                                              //f11086
      FREDLONG typedeflong = 6;                                                 //f11086
      typedefretint = testtypedef(typedefint, typedefchar, typedeflong);        //f11086
      break;
   case 122:
      /*-------------------------------------------------------------------*/
      /* Test structure in register                                        */
      /*-------------------------------------------------------------------*/
      ;
      int inttest = 0;                                                          //f11086

      struct taskret structret;                                                 //f11086
      structret.x = 5;                                                          //f11086
      structret.a  = 'A';                                                       //f11086

      struct taskparm1 structparm1;                                             //f11086
      structparm1.b = 'B';                                                      //f11086
      structparm1.c = 256;                                                      //f11086

      struct taskparm2 structparm2;                                             //f11086
      structparm2.d = 512;                                                      //f11086
      structparm2.e = 1024;                                                     //f11086

      struct taskparm3 structparm3;                                             //f11086
      structparm3.f = 15;                                                       //f11086
      structparm3.g = 'G';                                                      //f11086
      structparm3.h = 2048;                                                     //f11086

      struct taskparm4 structparm4;                                             //f11086
      structparm4.j = 'J';                                                      //f11086
      structparm4.k = 'K';                                                      //f11086
      structparm4.l = 'K';                                                      //f11086
      structparm4.m = 'K';                                                      //f11086
      structparm4.n = 'K';                                                      //f11086
      structparm4.o = 'K';                                                      //f11086
      structparm4.p = 'K';                                                      //f11086

      structret = teststructinreg(structparm1, structparm2,
                                  structparm3, structparm4);                    //f11086
      inttest =  teststructinreg2(structparm1, structparm2,
                                  structparm3, structparm4);                    //f11086
      break;
   case 123:
      /*-------------------------------------------------------------------*/
      /* Test structure in register                                        */
      /*-------------------------------------------------------------------*/
      ;
      int intchartest = 0;                                                      //f11086

      struct taskret structret22;                                               //f11086
      structret.x = 5;                                                          //f11086
      structret.a  = 'A';                                                       //f11086

      struct taskcharparm1 structcharparm1;                                     //f11086
      structcharparm1.jj = 'B';                                                 //f11086

      struct taskcharparm2 structcharparm2;                                     //f11086
      structcharparm2.jjj = 'C';                                                //f11086
      structcharparm2.kkk = 2;                                                  //f11086

      struct taskcharparm3 structcharparm3;                                     //f11086
      structcharparm3.jjjj = 'D';                                               //f11086
      structcharparm3.kkkk = 'E';                                               //f11086

      struct taskcharparm4 structcharparm4;                                     //f11086
      structcharparm4.jjjjj = 'F';                                              //f11086
      structcharparm4.kkkkk = 'G';                                              //f11086
      structcharparm4.sssss = 4;                                                //f11086

      struct taskcharparm66 structcharparm66;                                   //f11086
      structcharparm66.j1 = 'A';                                                //f11086
      structcharparm66.j2 = 'B';                                                //f11086
      structcharparm66.j3 = 'C';                                                //f11086
      structcharparm66.j4 = 'D';                                                //f11086
      structcharparm66.j5 = 'E';                                                //f11086
      structcharparm66.j6 = 'F';                                                //f11086

      struct taskcharparm55 structcharparm55;                                   //f11086
      structcharparm55.j1 = 'G';                                                //f11086
      structcharparm55.j2 = 'H';                                                //f11086
      structcharparm55.j3 = 'I';                                                //f11086
      structcharparm55.j4 = 'J';                                                //f11086
      structcharparm55.j5 = 'K';                                                //f11086

     struct taskcharparm44 structcharparm44;                                    //f11086
      structcharparm44.j1 = 'L';                                                //f11086
      structcharparm44.j2 = 'M';                                                //f11086
      structcharparm44.j3 = 'N';                                                //f11086
      structcharparm44.j4 = 'O';                                                //f11086

      struct taskcharparm33 structcharparm33;                                   //f11086
      structcharparm33.j1 = 'P';                                                //f11086
      structcharparm33.j2 = 'Q';                                                //f11086
      structcharparm33.j3 = 'R';                                                //f11086

      struct taskcharparm22 structcharparm22;                                   //f11086
      structcharparm22.j1 = 'S';                                                //f11086
      structcharparm22.j2 = 'T';                                                //f11086

      struct taskcharparm11 structcharparm11;                                   //f11086
      structcharparm11.j1 = 'U';                                                //f11086

      intchartest =  teststructinreg3(structcharparm1, structcharparm2,
                                     structcharparm3, structcharparm4);         //f11086
      structret22 =  teststructinreg55(structcharparm55, structcharparm44,
                                       structcharparm33, structcharparm22,
                                       structcharparm11);                       //f11086
      structret22 =  teststructinreg66(structcharparm66, structcharparm55,
                                       structcharparm44, structcharparm33,
                                       structcharparm22, structcharparm11);     //f11086
      structret22 =  teststructinreg66a(structcharparm22, structcharparm11,
                                       structcharparm22, structcharparm11,
                                       structcharparm22, structcharparm11);     //f11086
      break;
   default:
       bad_case_msg("QME1", test_case);                                         //C64JK
       exit(1);                                                                 //C64JK
   }

   /* Force a dump with a divide by zero, but only do so for test cases 100+ */ //f11086
   /* otherwise the dump would occur twice,  which we are not interested in. */ //f11086
   if (test_mode == DUMP_YES && test_case >= 100 ) {
     test_case = test_case / zero;                                              //f11086
   }
   return 0;
}
