/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* MODULE NAME..... QME3                                             */
/* DESCRIPTION..... This module serves as an entry point using the   */
/*                  main() function for the Shared Object Linkage    */
/*                  Driver (SOLD) test cases.                        */
/*                                                                   */
/* FUNCTION... QME3 is a function called by test cases               */
/*                     7, 10, 57                                     */
/*             of the SOLD driver. The test case number is passed to */
/*             this function through the parameter array argv and    */
/*             placed into the ECB workarea field EBW000.            */
/*                                                                   */
/*             qme3a is a function that takes six integer arguments  */
/*             and subtracts the product of the last three arguments */
/*             from the product of the first three.  The result is   */
/*             then returned. The result should be zero for normal   */
/*             return.                                               */
/*                                                                   */
/* RESTRICTIONS....                                                  */
/*                                                                   */
/* INPUT.....                                                        */
/*                                                                   */
/* NORMAL RETURN...                                                  */
/*                                                                   */
/* ERROR RETURN....                                                  */
/*                                                                   */
/* EXTERNAL ROUTINES.....                                            */
/*                                                                   */
/*********************************************************************/
#include <tpf/tpfapi.h>
#include <stdio.h>
#include <stdlib.h>
#include <c_sold.h>

void QCE3(void);                                                                //C64JK
void QSB6(struct TPF_regs *);                                                   //C64JK

int qme3a(int i1, int i2, int i3, int i4, int i5, int i6)
{
   int retval = (i1 * i2 * i3) - (i4 * i5 * i6);                                //C64JK

   if( retval != 0 ) {                                                          //C64JK
     bad_param_msg("QME3", test_case);                                          //C64JK
   }
   QCE3();                                                                      //C64JK

   return retval;                                                               //C64JK
}

int main(int argc, char **argv)
{
   test_case = atoi(argv[1]);                                                   //C64JK
   test_mode = atoi(argv[2]);                                                   //d10433f_1
   struct TPF_regs regs;                                                        //C64JK
   struct pat *pat_addr;                                                        //d10433f_1
   char prog_name[PAT_PROG_NAME_LEN+PAT_VERS_CODE_LEN+1] = {0};                 //d10875

   if (test_mode == SHOW_VER)  {                                                //f5555
      pat_addr = (void*) tpf_ecbp3()->ce3patg;                                  //f12472
      memcpy(prog_name, pat_addr->patname, PAT_PROG_NAME_LEN+PAT_VERS_CODE_LEN);//d10875
      show_prog_version(prog_name, pat_addr->patacnum);                         //f12305
   }

   switch(test_case) {
   case 7:
       if( qme3a(24, 10, 33, 30, 22, 12) != 0 ) {                               //C64JK
          bad_return_msg("QME3", "QME3", test_case);                            //C64JK
       }
       break;
   case 10:
       if( qsc1b('a', 'b', 'c', 'd', 'e', 'f', qsc3a) != 'P' ) {                //C64JK
          bad_return_msg("QME3", "QSC1", test_case);                            //C64JK
       }
       break;
   case 57:
       /* Set up TPF_regs */
       regs.r0 = 2003;                                                          //C64JK
       regs.r1 = -1000;                                                         //C64JK
       regs.r2 = 600000;                                                        //C64JK
       regs.r3 = -503599;                                                       //C64JK
       regs.r4 = -96000;                                                        //C64JK
       regs.r5 = 2602;                                                          //C64JK
       regs.r6 = (long)&regs.r0;                                                //C64JK
       QSB6(TPF_regs_Call64(&regs));                                            //C64JK
       /* Check results */
       if( regs.r7 != regs.r0 )                                                 //C64JK
          bad_return_msg("QME3", "QSB6", test_case);                            //C64JK
       break;
   default:
       bad_case_msg("QME3", test_case);                                         //C64JK
   }
return 0;
}
