         PRINT NOGEN
         BEGIN NAME=QSB4,IBM=NO,AMODE=64,BASE=(R1,R7,R8)
***********************************************************************
*          Licensed Materials - Property of IBM                       *
*          "Restricted Materials of IBM"                              *
*          5748-T15                                                   *
*          (C) Copyright IBM Corp. 2006                               *
*          All Rights Reserved                                        *
*          US Government Users Restricted Rights - Use, duplication   *
*          or disclosure restricted by GSA ADP Schedule Contract      *
*          with IBM Corp.                                             *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
*  Change History                                                     *
*                                                                     *
*                                                                     *
*                                                                     *
***********************************************************************
*                                                                     *
* MODULE NAME..... QSB4                                               *
* DESCRIPTION..... Assembler program, part of QSB1 BSO                *
*                                                                     *
* FUNCTION... 64-bit assembler program to test out linkages           *
*                                                                     *
*    This function represents a 64-bit assembler program with         *
*    3 program base registers, is greater than 4K in size, and no     *
*    transfer vectors.                                                *
*                                                                     *
*    This module contains code for test cases:                        *
*      29, 59                                                         *
*                                                                     *
*    For more information on these test cases, see qsca.c.            *
*                                                                     *
* MODULE ATTRIBUTES..                                                 *
*   TYPE.......... 'E'                                                *
*   PROCESSOR..... SHARED                                             *
*   DEPENDENCY.... IS IT OBJECT SHIPPABLE ??   NO                     *
*   ENVIRONMENT... BASE, ISO-C                                        *
*   ENTRY POINT... LABEL(S)                     (ONE PER E.P.)        *
*                                                                     *
* ECB-CONVENTIONS.......(DESCRIBE ALL INTERNAL ECB USAGE)             *
*                                                                     *
* -  TEST CASE NUMBER IS IN EBW000 FOR ALL SOLD SEGMENTS              *
*                                                                     *
* REGISTER-CONVENTIONS..(DESCRIBE ALL INTERNAL REGISTER USAGE)        *
*                                                                     *
***********************************************************************
* INTERFACE REQUIREMENTS:                                             *
*                                                                     *
* -  TEST CASE NUMBER IS IN EBW000 FOR ALL SOLD SEGMENTS              *
*                                                                     *
* DEPENDENCIES... DESCRIBE EXTERNAL EVENTS AFFECTING EXECUTION        *
* RESTRICTIONS... DESCRIBE ANY ENVIRONMENTAL OR CONFIGURATION         *
*                 ITEMS                                               *
*                                                                     *
*  ECB          * INPUT..              * OUTPUT..                     *
* --------------*----------------------*----------------------        *
* WORK AREA.... *                      *                              *
* DATA LEVELS.. *                      *                              *
* REGISTERS.... *                      *                              *
*                                                                     *
***********************************************************************
         SPACE
         LGHI  R11,-10      Corrupt R11 and R12 - you are supposed to
         LGHI  R12,-20        be able to use them
         L     R14,EBW000   Load the test case number
         CHI   R14,29       Is it test case 29?
         BE    CASE29       Yes, go do it
         CHI   R14,59       Case 59?
         BE    CASE59       Yes, go do it
         CHI   R14,12       Case 12?                                    C64CF
         BE    CASE12       Yes, go do it                               C64CF
         B     CASEERR      No, branch to error
         SPACE 1

CASE29   DS    0H
         BAS   R15,PARMVAL  Check parameters
         B     CASE29A      Branch to second base register area
         SPACE 1
PADDING  DC    4200C'X'     Pad 4200 X's to get to the next 4K block

***********************************************************************
PARMVAL  DS    0H
         LG    R14,8(R6)   Load the first value into R14 (Note that R1 X
                           is a base reg, so load it from addr in R6
         AGR   R14,R2      Add
         AGR   R14,R3        in
         AGR   R14,R4         remaining
         AGR   R14,R5          registers
         CGR   R0,R14      Is the sum what is expected?
         BNE   PARMERR     No, issue error
         CG    R0,0(R6)    Does the address in R6 point to the sum?
         BNE   PARMERR     No, issue error
         BR    R15

CASE29A  DS    0H
         B     CASE29B     Branch to third base register area
         SPACE 1
PADDING2 DC    4500C'Y'     Pad 4500 Y's to get to the next 4K block

***********************************************************************
CASE29B  DS  0H
* In this last 4K block, R8 is now the base register
         LG    R1,8(R6)    Restore R1
         L     R7,=C'SOLD' Restore R7
         ENTRC QSB6        Enter QSB6
         BACKC ,

CASE12   DS    0H                                                       C64CF
CASE59   DS    0H
         BAS   R15,PARMVAL  Check parameters                            C64CF

* Prepare for CALLC
         CPROC RETURN=i,qsc2b,(s,s,us,us)
         LMG   R0,R3,CONST59      Set up parameters
         CALLC qsc2b(R0,R1,R2,R3) Make C call
         MVC   EBX000(4),=C'QSC2' Prep for error routine
         CHI   R15,1              Check to see if case passed
         JNE   RETERR             If not, issue error
         LMG   R0,R7,CASE59RET    Load return values
         BACKC ,

CASEERR  DS    0H
* Bad test case number.  Send error message and exit
         WTOPC TEXT='QSB4: INVALID TEST CASE NUMBER ..',               X
               SUB=(DECA,EBW000),                                      X
               LET=E,NUM=93,PREFIX=SOLD
         EXITC ,
         SPACE
PARMERR  DS    0H
         WTOPC TEXT='QSB4: PARAMETER PASSING ERROR IN TEST CASE ..',   X
               SUB=(DECA,EBW000),                                      X
               LET=E,NUM=92,PREFIX=SOLD
         EXITC
         SPACE
RETERR   DS    0H
         WTOPC TEXT='QSB1: BAD RETURN VALUE FROM .... FOR TEST CASE ..'X
               ,SUB=(CHARA,EBX000,DECA,EBW000),                        X
               LET=E,NUM=91,PREFIX=SOLD
         EXITC ,
         SPACE
***********************************************************************
*                                                                     *
* CONSTANTS & PARMLISTS                                               *
*                                                                     *
***********************************************************************
         SPACE 1
CONST59  DS  0FD
         DC  FD'33000'
         DC  FD'40000'
         DC  FD'50000'
         DC  FD'60000'
         SPACE 1
CASE59RET DS 0FD
         DC  FD'256'
         DC  FD'-1'
         DC  FD'-32333294280293'
         DC  FD'100000000000000'
         DC  FD'2051999'
         DC  FD'0'
         DC  FD'-4042003'
         DC  FD'6'
         SPACE 1
         LTORG
         FINIS
         END
