         PRINT NOGEN
         BEGIN NAME=QSBB,IBM=NO,AMODE=31,BASE=(R2,R7,R8)
***********************************************************************
*          Licensed Materials - Property of IBM                       *
*          "Restricted Materials of IBM"                              *
*          5748-T15                                                   *
*          (C) Copyright IBM Corp. 2006                               *
*          All Rights Reserved                                        *
*          US Government Users Restricted Rights - Use, duplication   *
*          or disclosure restricted by GSA ADP Schedule Contract      *
*          with IBM Corp.                                             *
***********************************************************************
*  This file is provided on an "as is" basis.  There are no           *
*  warranties, express or implied, including the implied warranties   *
*  of merchantability and fitness for a particular purpose.           *
*  IBM has no obligation to provide service, defect correction, or    *
*  any maintenance for the file.  IBM has no obligation to supply     *
*  any updates or enhancements for the file to you even if such are   *
*  or later become available.                                         *
***********************************************************************
*  Change History                                                     *
*                                                                     *
*                                                                     *
*                                                                     *
***********************************************************************
*                                                                     *
* MODULE NAME..... QSBB                                               *
* DESCRIPTION..... Assembler program, part of QSBA BSO                *
*                                                                     *
* FUNCTION... 31-bit assembler program to test out linkages           *
*                                                                     *
*    This function represents a 31-bit assembler program with         *
*    3 program base registers, is greater than 4K in size, and no     *
*    transfer vectors.                                                *
*                                                                     *
*    This module contains code for test cases:                        *
*      26, 27, 58                                                     *
*                                                                     *
*    For more information on these test cases, see qsca.c.            *
*                                                                     *
* MODULE ATTRIBUTES..                                                 *
*   TYPE.......... 'E'                                                *
*   PROCESSOR..... SHARED                                             *
*   DEPENDENCY.... IS IT OBJECT SHIPPABLE ??   NO                     *
*   ENVIRONMENT... BASE, ISO-C                                        *
*   ENTRY POINT... LABEL(S)                     (ONE PER E.P.)        *
*                                                                     *
* ECB-CONVENTIONS.......(DESCRIBE ALL INTERNAL ECB USAGE)             *
*                                                                     *
* -  TEST CASE NUMBER IS IN EBW000 FOR ALL SOLD SEGMENTS              *
*                                                                     *
* REGISTER-CONVENTIONS..(DESCRIBE ALL INTERNAL REGISTER USAGE)        *
*                                                                     *
***********************************************************************
* INTERFACE REQUIREMENTS:                                             *
*                                                                     *
* -  TEST CASE NUMBER IS IN EBW000 FOR ALL SOLD SEGMENTS              *
*                                                                     *
* DEPENDENCIES... DESCRIBE EXTERNAL EVENTS AFFECTING EXECUTION        *
* RESTRICTIONS... DESCRIBE ANY ENVIRONMENTAL OR CONFIGURATION         *
*                 ITEMS                                               *
*                                                                     *
*  ECB          * INPUT..              * OUTPUT..                     *
* --------------*----------------------*----------------------        *
* WORK AREA.... *                      *                              *
* DATA LEVELS.. *                      *                              *
* REGISTERS.... *                      *                              *
*                                                                     *
***********************************************************************
         SPACE
         L     R14,EBW000   Load the test case number
         CHI   R14,26       Is it test case 26?
         JE    CASE26       Yes, go do it
         CHI   R14,27       Is it test case 27?
         JE    CASE27       Yes, go do it
         CHI   R14,58       Is it test case 58?
         JE    CASE58       Yes, go do it
         J     CASEERR
         SPACE 1

CASE26   DS    0H
         L     R14,8(R6)   Load the second value into R14 (Note that R2X
                           is a base reg, so load it from addr in R6
         BAS   R15,PARMVAL Check validity of parameters
         LM    R0,R6,CONST26   Load new constants into registers
         ENTRC QSBF        Enter QSBF (inter-BAL)
         BACKC ,

CASE27   DS    0H
         L     R14,8(R6)   Load the second value into R14 (Note that R2X
                           is a base reg, so load it from addr in R6
         JAS   R15,PARMVAL  Check validity of parameters
         J     CASE27A      Branch to second base register area
         SPACE 1
PADDING  DC    4200C'A'     Pad 4200 A's to get to the next 4K block

***********************************************************************
PARMVAL  DS    0H
         AR    R14,R1      Add
         AR    R14,R3        in
         AR    R14,R4         remaining
         AR    R14,R5          registers
         CR    R0,R14      Is the sum what is expected?
         JNE   PARMERR     No, issue error
         BR    R15         Yes, continue tests
         SPACE 1

CASE27A  DS    0H
         C     R0,0(R6)    Does the address in R6 point to the sum?
         JNE   PARMERR     No, issue error
         J     CASE27B     Branch to third base register area
         SPACE 1
PADDING2 DC    4500C'B'     Pad 4500 B's to get to the next 4K block

***********************************************************************
CASE27B  DS  0H
* In this last 4K block, R8 is now the base register
         L     R2,8(R6)    Restore R2
         L     R7,=C'SOLD' Restore R7
         ENTRC QSBE        Enter QSBE (inter-BAL)
* Check to see if the base registers were restored
* Also, change return register from 0 back to 7 and then use LREGSC
* to get it back.
         LR    R7,R0       Move result from QSBE to R7
         BACKC ,

CASE58   DS    0H
         L     R14,20(R6)  Load the second value into R14 (Note that R2X
                           is a base reg, so load it from addr in R6
         JAS   R15,PARMVAL Check parameters
         XR    R7,R7       Zero out register
         LLILL R7,C'P'     Test case passed
         BACKC ,

CASEERR  DS    0H
* Bad test case number.  Send error message and exit
         WTOPC TEXT='QSBB: INVALID TEST CASE NUMBER ..',               X
               SUB=(DECA,EBW000),                                      X
               LET=E,NUM=93,PREFIX=SOLD
         EXITC ,
         SPACE
PARMERR  DS    0H
* Bad parameters passed.  Send error message and exit
         WTOPC TEXT='QSBB: PARAMETER PASSING ERROR IN TEST CASE ..',   X
               SUB=(DECA,EBW000),                                      X
               LET=E,NUM=92,PREFIX=SOLD
         EXITC
         SPACE
RESTERR  DS    0H
* Base register not restored correctly.  Send error message and exit
         WTOPC TEXT='QSBB: BASE REGISTER NOT RESTORED CORRECTLY',      X
               LET=E,NUM=92,PREFIX=SOLD
         EXITC
         SPACE
***********************************************************************
*                                                                     *
* CONSTANTS & PARMLISTS                                               *
*                                                                     *
***********************************************************************
         SPACE 1
CONST26  DS  0F
         DC  F'780'      The last 3 values must add to this number
         DC  F'-32000032'
         DC  F'-2128'    but all 7 values must add up to 0
         DC  F'32000600'
         DC  F'-1300'
         DC  F'199'
         DC  F'1881'
         SPACE 1
         LTORG
         FINIS
         END
