/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* MODULE NAME..... QSC2 (qsc2b)                                     */
/* DESCRIPTION..... This module is a library of functions for the    */
/*                  Shared Object Linkage Driver (SOLD) test cases.  */
/*                  This is one function contained in the library.   */
/*                                                                   */
/* FUNCTION... qsc2b is a function called by test cases              */
/*                     2, 9, 59                                      */
/*             of the SOLD driver. The test case number is passed to */
/*             this function via the ECB workarea field EBW000.      */
/*             Two signed short ints and two unsigned short ints are */
/*             passed to this function. The signed short ints should */
/*             be passed in as large positive numbers, so that they  */
/*             are negative when the function actually uses them.    */
/*             The unsigned numbers should be large enough where     */
/*             they add up to a negative number.  The return value   */
/*             is 1 if successful, 0 otherwise.                      */
/*                                                                   */
/* RESTRICTIONS....                                                  */
/*                                                                   */
/* INPUT.....                                                        */
/*                                                                   */
/* NORMAL RETURN... 1                                                */
/*                                                                   */
/* ERROR RETURN....                                                  */
/*                                                                   */
/* EXTERNAL ROUTINES.....                                            */
/*                                                                   */
/*********************************************************************/
#include <c_sold.h>
#include <tpf/tpfapi.h>
#include <stdio.h>

void QCE3(void);                                                                //C64JK
void QSB9(struct TPF_regs *);                                                   //C64JK

int qsc2b( short int i1, short int i2, unsigned short int i3,
	   unsigned short int i4 )
{
  short int sum = i3 + i4;                                                      //C64JK
  struct TPF_regs regs;                                                         //C64JK

  if( (i1 > 0) || (i2 > 0) || (sum > 0) ) {                                     //C64JK
    bad_param_msg("QSC2", test_case);                                           //C64JK
  }

  switch(test_case) {
     case 2:
     case 9:
        /* End of the line */
        break;
     case 12:                                                                   //C64CF
     case 59:
        /* Set up parameters */
        regs.r0 = 568945021;                                                    //C64JK
        regs.r1 = 1051;                                                         //C64JK
        regs.r2 = 59865;                                                        //C64JK
        regs.r3 = 897541;                                                       //C64JK
        regs.r4 = 23;                                                           //C64JK
        regs.r5 = 567986541;                                                    //C64JK
        regs.r6 = (long)&regs.r0;                                               //C64JK
        memcpy(&regs.r7, "\0\0\0\0SOLD", sizeof(regs.r7));                      //C64JK
        QSB9(TPF_regs_Call64(&regs));                                           //C64JK
        if((char)regs.r7 != 'P')                                                //C64JK
           bad_return_msg("QSC3", "QSB9", test_case);                           //C64JK
        break;
     default:
        bad_case_msg("QSC2", test_case);                                        //C64JK
  }

  return 1;
}
