/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* MODULE NAME..... QSC3 (qsc3a)                                     */
/* DESCRIPTION..... This module is a library of functions for the    */
/*                  Shared Object Linkage Driver (SOLD) test cases.  */
/*                  This is one function contained in the library.   */
/*                                                                   */
/* FUNCTION... qsc3a is a function called by test cases              */
/*                     3, 8, 10, 58                                  */
/*             of the SOLD driver. The test case number is passed to */
/*             this function via the ECB workarea field EBW000.      */
/*             Six characters and one character pointer are passed   */
/*             to this function. The six chars, when concatenated,   */
/*             must be the same as the pointer passed in. The return */
/*             value is 'P' if successful, 'F' otherwise.            */
/*                                                                   */
/* RESTRICTIONS....                                                  */
/*                                                                   */
/* INPUT.....                                                        */
/*                                                                   */
/* NORMAL RETURN... 'P'                                              */
/*                                                                   */
/* ERROR RETURN....                                                  */
/*                                                                   */
/* EXTERNAL ROUTINES.....                                            */
/*                                                                   */
/*********************************************************************/
#include <tpf/tpfapi.h>
#include <stdio.h>
#include <string.h>
#include <c_sold.h>

void QCE2(void);                                                                //C64JK
void QSBB(struct TPF_regs *);                                                   //C64JK

char qsc3a( char c1, char c2, char c3, char c4, char c5, char c6,
	    char * str )
{
  char input[6] = {c1, c2, c3, c4, c5, c6};                                     //C64JK
  struct TPF_regs regs;                                                         //C64JK

  struct pat *pat_addr;                                                         //d10433f_1
  char prog_name[PAT_PROG_NAME_LEN+PAT_VERS_CODE_LEN+1] = {0};                  //d10875

  if (test_mode == SHOW_VER)  {                                                 //f5555
     pat_addr = (void*) tpf_ecbp3()->ce3patg;                                   //f12472
     memcpy(prog_name, pat_addr->patname, PAT_PROG_NAME_LEN+PAT_VERS_CODE_LEN); //d10875
     show_prog_version(prog_name, pat_addr->patacnum);                          //f12305
  }

  // String comparison against str
  if( strncmp(str, input, 6) ) {                                                //C64JK
     bad_param_msg("QSC3", test_case);                                          //C64JK
  }

  switch(test_case) {
  case 3:
  case 10:
     /* End of the line. Do nothing */
     break;
  case 8:
    QCE2();                                                                     //C64JK
    break;
  case 58:
     /* Set up parameters */
     regs.r0 = -64080;                                                          //C64JK
     regs.r1 = -62199;                                                          //C64JK
     regs.r2 = -7528;                                                           //C64JK
     regs.r3 = 7528;                                                            //C64JK
     regs.r4 = -51802;                                                          //C64JK
     regs.r5 = 49921;                                                           //C64JK
     regs.r6 = (long)&regs.r0;                                                  //C64JK
     QSBB(&regs);                                                               //C64JK
     /* Check return value */
     if( (char)regs.r7 != 'P' )                                                 //C64JK
        bad_return_msg("QSC3", "QSBB", test_case);                              //C64JK
     break;
  default:
    bad_case_msg("QSC3", test_case);                                            //C64JK
  }

  return 'P';                                                                   //C64JK
}
