/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*   f11565     2/2/05     Use subsystem unique token for gsysc for  */
/*                         the SOLD driver control block.            */
/*   f9127      2/24/05    Capable of stopping SOLD in continuous    */
/*                         mode on a specific istream.               */
/*   d13279     9/19/05    Minimize the number of ECBs created by    */
/*                         the continuous mode activites.            */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/** END OF APAR HISTORY **********************************************/
/*                                                                   */
/* SEGMENT NAME....  QSCB                                            */
/*                                                                   */
/*   Load Module Name:  QSCB                                         */
/*                                                                   */
/* RELATED SEGMENTS. This is part of the SOLD test driver            */
/* PURPOSE.........  To create 'continuous' mode for SOLD via        */
/*                   CREMCs and CRETCs                               */
/*                                                                   */
/*                                                                   */
/* DESCRIPTION...                                                    */
/*                                                                   */
/* QSCB  is used to make the Shared Object Linkage Driver (SOLD)     */
/* continuous.                                                       */
/*                                                                   */
/* QSCA calls QSCB, which then does a ENTRC for each test case,      */
/* except test case 32 and 33. Test case 32 and 33 test ENTDC and    */
/* ENTNC, which do NOT return the caller here. These test cases will */
/* then tested by child ECBs created by CREMC here.                  */
/* After each test case has been initiated and tested, QSCB then     */
/* does a CRETC back to itself to start things over again.           */
/*                                                                   */
/* NOTES:                                                            */
/*                                                                   */
/* MODULE ATTRIBUTES..                                               */
/*   TYPE..........  Shared Object                                   */
/*   PROCESSOR.....  shared                                          */
/*   ENVIRONMENT...  BASE, ISO-C                                     */
/*   ENTRY POINT...  LABEL(S)                     (ONE PER E.P.)     */
/*                                                                   */
/* ECB-CONVENTIONS.......(DESCRIBE ALL INTERNAL ECB USAGE)           */
/*                                                                   */
/* The test case number is kept in the ECB in EBW000 for all SOLD    */
/* segments.                                                         */
/*                                                                   */
/*********************************************************************/
/* INTERFACE REQUIREMENTS:                                           */
/*                                                                   */
/* The test case number is kept in the ECB in EBW000 for all SOLD    */
/* segments.                                                         */
/*                                                                   */
/* For each CREMC, an ECB work area is set up in order to pass       */
/* pertinent information to QSCC.                                    */
/*                                                                   */
/* The work area is off data level 0 (D0).                           */
/* The work area starts at EBW000.                                   */
/*                                                                   */
/* -  test case number      (EBW000-EBW003)                          */
/* -  terminal address      (EBROUT)                                 */
/* -  CPUID                 (CEICPD)                                 */
/* -  mode continuous flag  (EBW008)                                 */
/* -  percent of resource control activity  (EBW096-EBW099)          */
/* -  resource control message option       (EBW096-EBW099)          */
/*                                                                   */
/* DEPENDENCIES... None.                                             */
/*                                                                   */
/* RESTRICTIONS... NONE                                              */
/*                                                                   */
/*               * INPUT..              * OUTPUT..                   */
/* --------------*----------------------*----------------------------*/
/* PARAMETERS... * NONE                 * none                       */
/*               *                      *                            */
/*               *                      *                            */
/* ECB WORK AREA *                      *                            */
/* DATA LEVELS.. *                      *                            */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* EXTERNAL-REFERENCES..   none                                      */
/*      ROUTINES....  (SUBROUTINES, UTILITIES, ETC.)                 */
/*      DATA AREAS..  EBW000                                         */
/*                                                                   */
/*                                                                   */
/* EXIT-NORMAL.. DESCRIBE STANDARD EXIT(S)  (THOSE EXPECTED)         */
/*     -ERROR... NONE                                                */
/*                                                                   */
/*********************************************************************/

#include <stdlib.h>
#include <stdio.h>

#include <tpf/tpfapi.h>
#include <tpf/c_eb0eb.h>
#include <tpf/sysapi.h>

#include <c_sold.h>

static void start_test_case(struct sold_cntl *, int *, int);                    //C64CF

void QSCB()
{
  char header_buffer[BUFNO];       /* fill in text for output msg          */
  struct sold_cntl *ctl_ptr;       /* Pointer to SOLD control structure    */
  long size;
  int  count;                                                                   //C64CF

  char   sold_token[9];                                                         //F11565
  THIS_SS_SOLD_CNTL_TOKEN(sold_token);                                          //F11565

  ctl_ptr = (struct sold_cntl *)tpf_fsysc(sold_token, &size);                   //C64CF
  if (ctl_ptr == NULL) {                                                        //C64CF
     /* control block not found, error and exit */
     wtopc_insert_header(header_buffer, "SOLD", 96, 'E', WTOPC_SYS_TIME);       //C64CF
     wtopc("SOLD CONTROL BLOCK NOT FOUND",                                      //C64CF
               WTOPC_EBROUT, WTOPC_NO_CHAIN, header_buffer);                    //C64CF
     exit(-1);                                                                  //C64CF

  } else if (ctl_ptr->IS_stat[(ecbptr()->ce1isn)-1].status == STOP) {           //F9127
     /* The SOLD driver is stopping.  Stop this instance */
     tpf_stpoc(TPF_STPOC_ON, NULL);                                             //C64CF

     /* Lock the control block */
     lockc(&(ctl_ptr->lock), LOCK_O_SPIN);                                      //C64CF

     /* Decrement number of active instances */
     ctl_ptr->IS_stat[(ecbptr()->ce1isn)-1].drvr_cnt--;                         //C64CF
     ctl_ptr->total_cnt--;                                                      //C64CF

     /* Unlock the control block and exit */
     unlkc(&(ctl_ptr->lock));                                                   //C64CF
     tpf_stpoc(TPF_STPOC_OFF, NULL);                                            //C64CF
     exit(0);                                                                   //C64CF
  }                                                                             //C64CF


  /* Start new test cases */
  count = 0;                                                                    //C64CF

  /* Now do the CREMCs for each test case                         */
  start_test_case(ctl_ptr, &count, 1);                                          //C64CF

  start_test_case(ctl_ptr, &count, 2);                                          //C64CF

  start_test_case(ctl_ptr, &count, 3);                                          //C64CF

  start_test_case(ctl_ptr, &count, 4);                                          //C64CF

  start_test_case(ctl_ptr, &count, 5);                                          //C64CF

  start_test_case(ctl_ptr, &count, 6);                                          //C64CF

  start_test_case(ctl_ptr, &count, 7);                                          //C64CF

  start_test_case(ctl_ptr, &count, 8);                                          //C64CF

  start_test_case(ctl_ptr, &count, 9);                                          //C64CF

  start_test_case(ctl_ptr, &count, 10);                                         //C64CF

  start_test_case(ctl_ptr, &count, 12);                                         //C64CF

  start_test_case(ctl_ptr, &count, 26);                                         //C64CF

  start_test_case(ctl_ptr, &count, 27);                                         //C64CF

  start_test_case(ctl_ptr, &count, 28);                                         //C64CF

  start_test_case(ctl_ptr, &count, 29);                                         //C64CF

  start_test_case(ctl_ptr, &count, 30);                                         //C64CF

  start_test_case(ctl_ptr, &count, 31);                                         //C64CF

  start_test_case(ctl_ptr, &count, 32);                                         //C64CF

  start_test_case(ctl_ptr, &count, 33);                                         //C64CF

  start_test_case(ctl_ptr, &count, 34);                                         //C64CF

  start_test_case(ctl_ptr, &count, 51);                                         //C64CF

  start_test_case(ctl_ptr, &count, 52);                                         //C64CF

  start_test_case(ctl_ptr, &count, 53);                                         //C64CF

  start_test_case(ctl_ptr, &count, 54);                                         //C64CF

  start_test_case(ctl_ptr, &count, 55);                                         //C64CF

  start_test_case(ctl_ptr, &count, 56);                                         //C64CF

  start_test_case(ctl_ptr, &count, 57);                                         //C64CF

  start_test_case(ctl_ptr, &count, 58);                                         //C64CF

  start_test_case(ctl_ptr, &count, 59);                                         //C64CF

  start_test_case(ctl_ptr, &count, 60);                                         //C64CF

  start_test_case(ctl_ptr, &count, 61);

  /* Increment number of cases executed */
  tpf_stpoc(TPF_STPOC_ON, NULL);                                                //C64CF
  /* Since this count field in the control block is i-stream unique, */
  /* we do not need to lock to update this field since we will not   */
  /* give up control (we can't be suspended or timesliced)           */
  ctl_ptr->IS_stat[(ecbptr()->ce1isn)-1].case_cnt += count;                     //C64CF
  tpf_stpoc(TPF_STPOC_OFF, NULL);                                               //C64CF

  /* Done with all the test cases.  Now CRETC back to myself.   */
  cretc(CRETC_SECONDS | CRETC_1052, QSCB, CRET_INTERVAL, NULL);                 //C64CF

  return;                                                                       //C64CF
}


static void start_test_case(struct sold_cntl *ctl_ptr, int *tcount, int tcase)
{
  int    defer_count;
  char   header_buffer[BUFNO];       /* fill in text for output msg          */
  struct savedata savedata_ptr;

  /* Only start a new test case if we are above system shutdown levels */
  defer_count = 0;
  if (lodic_ext(LODIC_CHECK + LODIC_BATCH, NULL) != 1) {
     if (defer_count >= 200) {
        wtopc_insert_header(header_buffer, "SOLD", 94, 'W', WTOPC_SYS_TIME);
        wtopc("SYSTEM BELOW LODIC BATCH SHUTDOWN LEVELS - SOLD IN DEFER LOOP",
               WTOPC_EBROUT, WTOPC_NO_CHAIN, header_buffer);                    //C64CF
        defer_count = 0;
     }

     if (*tcount > 0) {
        /* If we have a test count > 0, increment the in the control block */
        tpf_stpoc(TPF_STPOC_ON, NULL);
        /* Since this count field in the control block is i-stream unique, */
        /* we do not need to lock to update this field since we will not   */
        /* give up control (we can't be suspended or timesliced)           */
        ctl_ptr->IS_stat[(ecbptr()->ce1isn)-1].case_cnt += *tcount;
        tpf_stpoc(TPF_STPOC_OFF, NULL);

        /* set the counter back to zero */
        *tcount = 0;
     }

     if (ctl_ptr->IS_stat[(ecbptr()->ce1isn)-1].status == STOP) {
        /* The status is changed to stop    */
        /* Stop this instance of the driver */
        tpf_stpoc(TPF_STPOC_ON, NULL);

        /* Lock the control block */
        lockc(&(ctl_ptr->lock), LOCK_O_SPIN);

        /* Decrement number of active instances */
        ctl_ptr->IS_stat[(ecbptr()->ce1isn)-1].drvr_cnt--;
        ctl_ptr->total_cnt--;

        /* Unlock the control block and exit */
        unlkc(&(ctl_ptr->lock));
        tpf_stpoc(TPF_STPOC_OFF, NULL);
        exit(0);
     }

     defer_count++;
     defrc();
  }

  /* start test case */
  savedata_ptr.save_tcnum = tcase;
  savedata_ptr.save_cont_flag = CONT_YES;

  if ((tcase == 32) || (tcase == 33))
     /* Test case 32 and 33 do NOT return back here. */
     /* So we create new ECBs to test those cases.   */
     cremc(sizeof(struct savedata), &savedata_ptr, QSCC);
  else
  {
     /* All other cases return back here.         */
     /* Setup the parms in EBW000 area and enter  */
     /* to QSCC with return.                      */
     memcpy(&(ecbptr()->ebw000), &savedata_ptr, sizeof(struct savedata));
     entrc("QSCC");
  }

  (*tcount)++;

  /* give up control before starting another test case */
  /* to prevent CTL-10                                 */
  tpf_yieldc(YIELDC_READY_LIST);

  return;
}
