/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/*********************************************************************/
/* Change History                                                    */
/* BKopac             05/13/04               Added test case 14 for  */         //CRK7298
/*                                           very long function names*/
/*                                           greater than 2000 chars */
/* NWang              09/10/04               Add test cases 15 - 22  */
/*                                           for Loader Dynamic      */
/*                                           Linkage                 */
/* JKotski            1/28/2005              Added test cases 100 -  */
/*                                           123 for function trace  */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* MODULE NAME..... QSCC                                             */
/* DESCRIPTION..... This function is used for continuous             */
/*                  invocation of the Shared Object Linkage          */
/*                  Driver (SOLD).                                   */
/*                                                                   */
/* LEVEL........... VERSION 1  MODIFICATION 0                        */
/*                                                                   */
/* FUNCTION... QSCC is called by QSCB via CREMCs.                    */
/*             For each test case, QSCB kicks off a CREMC to this    */
/*             "mode continuous" section, which then obtains the     */
/*             information it needs from the ECB.  This area is      */
/*             pointed to by savedata_ptr.                           */
/*                                                                   */
/*             Once the needed information (test case number,        */
/*             CRAS settings, CPU ID) is put into the correct fields */
/*             the function SOLD_do_test_case is invoked.            */
/*                                                                   */
/*                                                                   */
/*             SOLD_do_test_case is called by QSCA and QSCC.         */
/*             The parameter for this function is the test case      */
/*             number, which is passed via test_case                 */
/*                                                                   */
/*             QSCA calls this function when the user invokes SOLD   */
/*             for an individual CASE number. QSCC calls it as part  */
/*             of its continuous mode operation.                     */
/*                                                                   */
/* RESTRICTIONS....                                                  */
/*                                                                   */
/* INPUT.....       VOID                                             */
/*                                                                   */
/* NORMAL RETURN... VOID                                             */
/*                                                                   */
/* ERROR RETURN....                                                  */
/*                                                                   */
/* EXTERNAL ROUTINES.....                                            */
/*                                                                   */
/*********************************************************************/
/* CHANGE ACTIVITY...                                                */
/*                                                                   */
/*        UPDATE BY          PLOG #   DATE   DESCRIPTION             */
/*        ---------          -------  ------ ------------            */
/*********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <tpf/c_eb0eb.h>
#include <tpf/tpfio.h>
#include <tpf/tpfregs.h>
#include <c_sold.h>
#include <tpf/tpfapi.h>
#include <errno.h>

char *start_msg1 = "TEST CASE ";
char *start_msg2 = " HAS COMPLETED";
char *start_msg3 = " HAS STARTED";

static void start_msg(int);
void QSB1(struct TPF_regs *);                                                   //C64CF
void QSB3(struct TPF_regs *);                                                   //C64JK
void QSB4(struct TPF_regs *);                                                   //C64JK
void QSB5(struct TPF_regs *);                                                   //C64CF
void QSBA(struct TPF_regs *);                                                   //C64CF
void QSBC(struct TPF_regs *);                                                   //C64JK
void QSBE(struct TPF_regs *);                                                   //C64JK
void QCE1(void);                                                                //C64JK
void QCE2(void);                                                                //C64JK
void QCE3(void);                                                                //C64JK
void call_non_existing_function(void);                                          //f4692

int test_mode_num = CONT_NO;                                                    //d10433f_1

void QSCC()
{
  struct savedata *savedata_ptr;                                                //C64JK

  savedata_ptr = (struct savedata *) &(ecbptr()->ebw000);                       //C64JK
  test_case = savedata_ptr->save_tcnum;      /* Get the test case number   */   //C64JK
  test_mode = savedata_ptr->save_cont_flag;  /* Set to continuous mode     */   //C64JK
  test_mode_num = savedata_ptr->save_cont_flag;                     //d10433f_1 //C64JK
  SOLD_do_test_case();                                                          //C64JK

  return;                                                                       //C64CF
}

void SOLD_do_test_case ()
{
   char test_msg[100];         /* Buffer for making system() call */            //C64JK
   int zero = 0;            /* for DUMP option (removes compiler warning) */

  /*--------------------------------------------------------------------*/
  /* Message variables                                                  */
  /*--------------------------------------------------------------------*/
  char header_buffer[BUFNO];
  char message_buf[BUFNO];     /* fill in text for output msg          */
  char *invalid_testcase = "INVALID TEST CASE NUMBER %d\n"
                           "  SEE QSCA.C FOR THE COMPLETE LIST OF VALID\n"
                           "  SOLD DRIVER TEST CASE NUMBERS";
  char *callee=NULL;
  struct TPF_regs regs;
  struct pat *pat_addr;                                                         //d10433f_1
  char prog_name[PAT_PROG_NAME_LEN+PAT_VERS_CODE_LEN+1] = {0};                  //d10875

  if (test_mode == SHOW_VER)  {                                                 //f5555
     test_mode_num = SHOW_VER;                                                  //d10433f_1
     pat_addr = (void *) tpf_ecbp3()->ce3patg;                                  //f12472
     memcpy(prog_name, pat_addr->patname, PAT_PROG_NAME_LEN+PAT_VERS_CODE_LEN); //d10875
     show_prog_version(prog_name, pat_addr->patacnum);                          //f12305
  }

  start_msg(test_case);

  /* Note: If you are adding a test case to this section, */
  /* you should probably also add it to the continuous    */
  /* mode section in qscb.c.                              */

  /* Find and execute the test case */
  switch (test_case)
   {
      case 1:
      case 4:
      case 51:
         callee = "QCE1";                                                       //C64JK
         QCE1();                                                                //C64JK
         break;
      case 2:
      case 61:
         callee = "QCE2";                                                       //C64JK
         QCE2();                                                                //C64JK
         break;
      case 3:
      case 52:
      case 62:
      case 63:
         callee = "QCE3";                                                       //C64JK
         QCE3();
         break;
      case 5:
      case 8:
      case 55:
      case 101:
      case 102:
      case 103:
      case 104:
      case 105:
      case 106:
      case 107:
      case 108:
      case 109:
      case 110:
      case 111:
      case 112:
      case 113:
      case 114:
      case 115:
      case 116:
      case 117:
      case 118:
      case 119:
      case 120:
      case 121:
      case 122:
      case 123:
          sprintf(test_msg, "QME1 %d %d", test_case, test_mode);                //f11086
          callee = "QME1";                                                      //C64JK
          system(test_msg);                                                     //C64JK
          break;                                                                //C64JK
      case 6:
      case 9:
      case 56:
          sprintf(test_msg, "QME2 %d %d", test_case, test_mode_num);//d10433f_1 //C64JK
          callee = "QME2";                                                      //C64JK
          system(test_msg);                                                     //C64JK
          break;                                                                //C64JK
      case 7:
      case 10:
      case 57:
          sprintf(test_msg, "QME3 %d %d", test_case, test_mode_num);//d10433f_1 //C64JK
          callee = "QME3";                                                      //C64JK
          system(test_msg);                                                     //C64JK
          break;                                                                //C64JK
      case 26:
      case 27:
      case 53:
         callee = "QSBA";                                                       //C64JK
         QSBA(NULL);                                                            //C64JK
         break;
      case 28:
      case 29:
      case 33:
      case 54:
         callee = "QSB1";                                                       //C64CF
         QSB1(NULL);                                                            //C64CF
         break;
      case 30:
         callee = "QSBC";                                                       //C64JK
         QSBC(NULL);                                                            //C64JK
         break;
      case 31:
         callee = "QSB5";                                                       //C64JK
         QSB5(NULL);                                                            //C64JK
         break;
      case 32:
      case 34:
         callee = "QSBE";                                                       //C64JK
         QSBE(NULL);                                                            //C64JK
         break;
      case 58:
      case 60:
         /* Set up parameters */
         regs.r0 = -5301750;                                                    //C64JK
         regs.r1 = 21354;                                                       //C64JK
         regs.r2 = 459873;                                                      //C64JK
         regs.r3 = -48972;                                                      //C64JK
         regs.r4 = -6302003;                                                    //C64JK
         regs.r5 = 567998;                                                      //C64JK

         /* The only difference between cases 58 and 60 is 31-bit vs 64-bit */
         if(test_case == 58) {                                                  //C64JK
            regs.r6 = (long)&regs.r0;                                           //C64CF
            memcpy(&regs.r7, "\0\0\0\0SOLD", sizeof(regs.r7));                  //C64JK
            callee = "QSB3";                                                    //C64JK
            QSB3(TPF_regs_Call64(&regs));                                       //C64JK
         }                                                                      //C64JK
         else {                                                                 //C64JK
            regs.r6 = ((long)&regs.r0) + 4;                                     //C64CF
            memcpy(&regs.r7, "\0\0\0\0SOLD", sizeof(regs.r7));                  //C64CF
            callee = "QSBE";                                                    //C64JK
            QSBE(&regs);                                                        //C64JK
            if ((regs.r0 != 0) ||                                               //C64CF
                (regs.r1 != -1) ||                                              //C64CF
                (regs.r2 != 42) ||                                              //C64CF
                (regs.r3 != 5000000) ||                                         //C64CF
                (regs.r4 != 1234567890) ||                                      //C64CF
                (regs.r5 != -1234567890) ||                                     //C64CF
                (regs.r6 != 2042004) ||                                         //C64CF
                (regs.r7 != 5)) {                                               //C64CF
                  bad_return_msg("QSCC", "QSBE", test_case);                    //C64CF
            }                                                                   //C64CF
         }                                                                      //C64CF
         break;
      case 59:
         /* Set up parameters */
         regs.r0 = 448384;                                                      //C64JK
         regs.r1 = 9634;                                                        //C64JK
         regs.r2 = 65;                                                          //C64JK
         regs.r3 = 463287;                                                      //C64JK
         regs.r4 = -12113;                                                      //C64JK
         regs.r5 = -12489;                                                      //C64JK
         regs.r6 = (long)&regs.r0;                                              //C64JK
         callee = "QSB4";                                                       //C64JK
         QSB4(TPF_regs_Call64(&regs));                                          //C64JK
         if ((regs.r0 != 256) ||                                                //C64CF
             (regs.r1 != -1) ||                                                 //C64CF
             (regs.r2 != -32333294280293) ||                                    //C64CF
             (regs.r3 != 100000000000000) ||                                    //C64CF
             (regs.r4 != 2051999) ||                                            //C64CF
             (regs.r5 != 0) ||                                                  //C64CF
             (regs.r6 != -4042003) ||                                           //C64CF
             (regs.r7 != 6)) {                                                  //C64CF
               bad_return_msg("QSCC", "QSB4", test_case);                       //C64CF
         }                                                                      //C64CF
         break;
      case 11:
           sprintf(message_buf," %s",strerror(CUSTOMER_UNDER_VALID_ERRNO));     //BE
           wtopc_insert_header(header_buffer, "SOLD", 11, 'I', NULL);           //BE
           wtopc(message_buf, WTOPC_EBROUT, WTOPC_NO_CHAIN, header_buffer);     //BE

           sprintf(message_buf," %s",strerror(CUSTOMER_FIRST_VALID_ERRNO));     //BE
           wtopc_insert_header(header_buffer, "SOLD", 11, 'I', NULL);           //BE
           wtopc(message_buf, WTOPC_EBROUT, WTOPC_NO_CHAIN, header_buffer);     //BE

           sprintf(message_buf," %s",strerror(CUSTOMER_MIDDLE_VALID_ERRNO));    //BE
           wtopc_insert_header(header_buffer, "SOLD", 11, 'I', NULL);           //BE
           wtopc(message_buf, WTOPC_EBROUT, WTOPC_NO_CHAIN, header_buffer);     //BE

           sprintf(message_buf," %s",strerror(CUSTOMER_LAST_VALID_ERRNO));      //BE
           wtopc_insert_header(header_buffer, "SOLD", 11, 'I', NULL);           //BE
           wtopc(message_buf, WTOPC_EBROUT, WTOPC_NO_CHAIN, header_buffer);     //BE

           sprintf(message_buf," %s",strerror(CUSTOMER_OVER_VALID_ERRNO));      //BE
           wtopc_insert_header(header_buffer, "SOLD", 11, 'I', NULL);           //BE
           wtopc(message_buf, WTOPC_EBROUT, WTOPC_NO_CHAIN, header_buffer);     //BE

           errno = CUSTOMER_UNDER_VALID_ERRNO;                                  //BE
           perror("The value of errno");                                        //BE
           errno = CUSTOMER_FIRST_VALID_ERRNO;                                  //BE
           perror("The value of errno");                                        //BE
           errno = CUSTOMER_MIDDLE_VALID_ERRNO;                                 //BE
           perror("The value of errno");                                        //BE
           errno = CUSTOMER_LAST_VALID_ERRNO;                                   //BE
           perror("The value of errno");                                        //BE
           errno = CUSTOMER_OVER_VALID_ERRNO;                                   //BE
           perror("The value of errno");                                        //BE
           break;                                                               //BE
      case 12:
         /* run some existing test cases through entrc() */

         /* Set up parameters */
         regs.r0 = 3987919;                                                     //C64CF
         regs.r1 = 9876;                                                        //C64CF
         regs.r2 = 123456;                                                      //C64CF
         regs.r3 = -48972;                                                      //C64CF
         regs.r4 = 6302003;                                                     //C64CF
         regs.r5 = -2398444;                                                    //C64CF
         regs.r6 = ((long)&regs.r0) + 4;                                        //C64CF
         memcpy(&regs.r7, "\0\0\0\0SOLD", sizeof(regs.r7));                     //C64CF
         callee = "QSBE";                                                       //C64CF
         entrc("QSBE", &regs);                                                  //C64CF
         if ((regs.r0 != 0) ||                                                  //C64CF
             (regs.r1 != -1) ||                                                 //C64CF
             (regs.r2 != 42) ||                                                 //C64CF
             (regs.r3 != 5000000) ||                                            //C64CF
             (regs.r4 != 1234567890) ||                                         //C64CF
             (regs.r5 != -1234567890) ||                                        //C64CF
             (regs.r6 != 2042004) ||                                            //C64CF
             (regs.r7 != 5)) {                                                  //C64CF
               bad_return_msg("QSCC", "QSBE", test_case);                       //C64CF
         }                                                                      //C64CF

         /* Set up parameters */
         regs.r0 = -32333294280293;                                             //C64CF
         regs.r1 =  65987654333001;                                             //C64CF
         regs.r2 = -98320947503294;                                             //C64CF
         regs.r3 = 0;                                                           //C64CF
         regs.r4 = 1171974;                                                     //C64CF
         regs.r5 = -2281974;                                                    //C64CF
         regs.r6 = (long)&regs.r0;                                              //C64CF
         callee = "QSB4";                                                       //C64CF
         entrc("QSB4", (TPF_regs_Call64(&regs)));                               //C64CF
         if ((regs.r0 != 256) ||                                                //C64CF
             (regs.r1 != -1) ||                                                 //C64CF
             (regs.r2 != -32333294280293) ||                                    //C64CF
             (regs.r3 != 100000000000000) ||                                    //C64CF
             (regs.r4 != 2051999) ||                                            //C64CF
             (regs.r5 != 0) ||                                                  //C64CF
             (regs.r6 != -4042003) ||                                           //C64CF
             (regs.r7 != 6)) {                                                  //C64CF
               bad_return_msg("QSCC", "QSB4", test_case);                       //C64CF
         }                                                                      //C64CF

         /* Call a BAL program without TPF_regs*/
         entrc("QSBA", NULL);                                                   //BE

         /* Call a CSO without any parameters */
         callee = "QCE3";                                                       //C64CF
         entrc("QCE3");                                                         //C64CF

         /* Call a CSO with parameters and a return value */
         /*        long QCE1(char, double, long, short, void *, int);  */
         if ((long)entrc("QCE1", 'F', 3.141526, 42000000000, -12345, "QCE1", 1234567890) != 42) {
            bad_return_msg("QSCC", "QCE1", test_case);                          //C64CF
         }                                                                      //C64CF
         break;

      case 13:
         LONG_FUNCTION_NAME();
         break;

      case 14:
         very_long_function_name(); /* test very long func names > 2000 chars*/ //CRK7298
         break;

      /***********************************************************/
      /*  Loaders' Dynamic Linkage error test cases (case 15-21) */
      /***********************************************************/
      case 15:   // BSO_CALLS_PROG_QQQQ                                         //f4692
           QSBE(NULL);                                                          //f4692
           break;                                                               //f4692

      case 16:   // CSO_CALLS_PROG_QQQQ                                         //f4692
           printf("QSCC: CSO CALLS NON EXISTING PROGRAM QQQQ - should get dump\n");//f4692
           entrc("QQQQ");                                                       //f4692
           break;                                                               //f4692

      case 17:   // BSO_CALLS_DATA_PROG_BXAX                                    //f4692
           QSBE(NULL);                                                          //f4692
           break;                                                               //f4692

      case 18:   // CSO_CALLS_DATA_PROG_BXAX                                    //f4692
           printf("QSCC: CSO CALLS DATA PROGRAM BXAX - should get dump\n");     //f4692
           entrc("BXAX");                                                       //f4692
           break;                                                               //f4692

      case 19:   // BSO_CALLS_NO_ENTRY_POINT_GTAL                               //f4692
           QSBE(NULL);                                                          //f4692
           break;                                                               //f4692

      case 20:   // CSO_CALLS_NO_ENTRY_POINT_GTAL                               //f4692
           printf("QSCC: CSO CALLS NO_ENTRY_POINT_GTAL - should get dump\n");   //f4692
           entrc("GTAL");                                                       //f4692
           break;                                                               //f4692

      case 21:   // CSO_CALLS_NON_EXIST_FUNCTION                                //f4692
           printf("QSCC: CSO calls a non-existent function\n");                 //f4692
           call_non_existing_function();                                        //f4692
           break;                                                               //f4692

      case 22:   // DO_CASE_15_TO_21                                            //f4692
           test_case = 15;                                                      //f4692
           cremc(sizeof(test_case), &test_case, QSBE);                          //f4692
           sleep(1);                                                            //f4692

           wtopc("QSCC: CSO CALLS NON EXISTING PROGRAM QQQQ - should get dump", //f4692
                  WTOPC_PRC, WTOPC_NO_CHAIN, WTOPC_NO_HEADER );                 //f4692
           cremc(sizeof(test_case), &test_case, QQQQ);                          //f4692
           sleep(1);                                                            //f4692

           test_case = 17;                                                      //f4692
           cremc(sizeof(test_case), &test_case, QSBE);                          //f4692
           sleep(1);                                                            //f4692

           wtopc("QSCC: CSO CALLS DATA PROGRAM BXAX - should get dump",         //f4692
                   WTOPC_PRC, WTOPC_NO_CHAIN, WTOPC_NO_HEADER );                //f4692
           cremc(sizeof(test_case), &test_case, BXAX);                          //f4692
           sleep(1);                                                            //f4692

           test_case = 19;                                                      //f4692
           cremc(sizeof(test_case), &test_case, QSBE);                          //f4692
           sleep(1);                                                            //f4692

           wtopc("QSCC: CSO CALLS NO_ENTRY_POINT_GTAL - should get dump",       //f4692
                   WTOPC_PRC, WTOPC_NO_CHAIN, WTOPC_NO_HEADER );                //f4692
           cremc(sizeof(test_case), &test_case, GTAL);                          //f4692
           sleep(1);

           wtopc("QSCC: CSO calls a non-existent function",                     //f4692
                   WTOPC_PRC, WTOPC_NO_CHAIN, WTOPC_NO_HEADER );                //f4692
           call_non_existing_function();                                        //f4692
           break;                                                               //f4692

      case 100:
         sprintf(test_msg, "QME4 %d", test_mode);                               //f11086
         callee = "QME4";                                                       //f11086
         system(test_msg);                                                      //f11086
         break;                                                                 //f11086
      default:
         sprintf(message_buf, invalid_testcase,test_case);
         wtopc_insert_header(header_buffer,"SOLD",94,'E',NULL);
         wtopc(message_buf,WTOPC_EBROUT,WTOPC_NO_CHAIN,header_buffer);
         exit(0);
   }

  if (test_case == 33 || test_case == 32) {                                     //C64CF
    /* test case was not supposed to return.  Issue error message  */
    sprintf(message_buf,"QSCC: UNEXPECTED RETURN FROM %s FOR TEST CASE %d",     //C64CF
       callee, test_case);                                                      //C64CF
    wtopc_insert_header(header_buffer, "SOLD", 90, 'E', NULL);                  //C64CF
    wtopc(message_buf, WTOPC_EBROUT, WTOPC_NO_CHAIN, header_buffer);            //C64CF
  }                                                                             //C64CF

  /* Force a dump with a divide by zero, but doing so for test cases 100+   */  //f11086
  /* would actually cause a duplicate dump, which we are not interested in. */  //f11086
  if (test_mode == DUMP_YES && test_case < 100 ) {                              //f11086
     test_case = test_case / zero;
  }

   if (test_mode == CONT_NO || test_mode == SHOW_VER)                           //f4692
  {
     sprintf(message_buf, "%s %d %s",start_msg1,test_case,start_msg2);
     wtopc_insert_header(header_buffer,"SOLD",4,WTOPC_INFO,NULL);
     wtopc(message_buf,WTOPC_EBROUT,WTOPC_NO_CHAIN,header_buffer);
  }


  return;
}


static void start_msg(int tcnum)
{
  char header_buffer[BUFNO];  /* fill in text for output msg          */
  char message_buf[BUFNO];

  /* if not in continuous mode, issue test case start message */
  if (test_mode != CONT_YES)  {
     sprintf(message_buf, "%s %d %s",start_msg1,tcnum,start_msg3);
     wtopc_insert_header(header_buffer,"SOLD",3,WTOPC_INFO,NULL);
     wtopc(message_buf,WTOPC_EBROUT,WTOPC_NO_CHAIN,header_buffer);
  }

  return;
}




