/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* RHung         f11565   2/2/05             Use subsystem unique    */
/*                 token for gsysc for the SOLD driver control block.*/
/* RHung         f9127   2/24/05             Capable of stopping     */
/*                 SOLD in continuous mode on a specific istream.    */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* MODULE NAME..... QSCD                                             */
/* DESCRIPTION..... This function is used for individual test case   */
/*                  invocation for the Shared Object Linkage         */
/*                  Driver (SOLD).                                   */
/*                                                                   */
/* FUNCTION... This source file contains utility functions to print  */
/*             out common error messages and exit the driver.        */
/*                                                                   */
/*                                                                   */
/* RESTRICTIONS....                                                  */
/*                                                                   */
/* INPUT.....       VOID                                             */
/*                                                                   */
/* NORMAL RETURN... VOID                                             */
/*                                                                   */
/* ERROR RETURN....                                                  */
/*                                                                   */
/* EXTERNAL ROUTINES.....                                            */
/*                                                                   */
/*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <tpf/tpfio.h>
#include <tpf/tpfapi.h>
#include <tpf/sysapi.h>
#include <tpf/i_ecb3.h>                                                         //f5555
#include <tpf/c_idspat.h>                                                       //f5555

#include <c_sold.h>

void bad_case_msg( char * src, int tcase )
{
   char header_buffer[BUFNO];   /* fill in text for header of output msg */
   char message_buf[BUFNO];     /* fill in text for output msg           */

   sprintf(message_buf, "%s: INVALID TEST CASE NUMBER %d", src, tcase);         //C64JK
   wtopc_insert_header(header_buffer, "SOLD", 93, 'E', NULL);                   //C64JK
   wtopc(message_buf, WTOPC_EBROUT, WTOPC_NO_CHAIN, header_buffer);             //C64JK
   exit(0);                                                                     //C64JK
}

void bad_param_msg( char * src, int tcase )
{
   char header_buffer[BUFNO];   /* fill in text for header of output msg */
   char message_buf[BUFNO];     /* fill in text for output msg           */

   sprintf(message_buf, "%s: PARAMETER PASSING ERROR IN TEST CASE %d",          //C64JK
           src, tcase);                                                         //C64JK
   wtopc_insert_header(header_buffer, "SOLD", 92, 'E', NULL);                   //C64JK
   wtopc(message_buf, WTOPC_EBROUT, WTOPC_NO_CHAIN, header_buffer);             //C64JK
   exit(0);                                                                     //C64JK
}

void bad_return_msg( char * src, char * culprit, int tcase )
{
   char header_buffer[BUFNO];   /* fill in text for header of output msg */
   char message_buf[BUFNO];     /* fill in text for output msg           */

   sprintf(message_buf, "%s: BAD RETURN VALUE FROM %s FOR TEST CASE %d",        //C64JK
           src, culprit, tcase);                                                //C64JK
   wtopc_insert_header(header_buffer, "SOLD", 91, 'E', NULL);                   //C64JK
   wtopc(message_buf, WTOPC_EBROUT, WTOPC_NO_CHAIN, header_buffer);             //C64JK
   exit(0);
}

void bad_static_msg( char * src, int tcase )
{
   char header_buffer[BUFNO];   /* fill in text for header of output msg */
   char message_buf[BUFNO];     /* fill in text for output msg           */

   sprintf(message_buf, "%s: STATIC VARIABLE HAS WRONG VALUE IN TEST CASE %d",  //C64JK
           src, test_case);                                                     //C64JK
   wtopc_insert_header(header_buffer, "SOLD", 90, 'E', NULL);                   //C64JK
   wtopc(message_buf, WTOPC_EBROUT, WTOPC_NO_CHAIN, header_buffer);             //C64JK
   exit(0);                                                                     //C64JK
}

void status_msg(void)
{
    char header_buffer[BUFNO];   /* fill in text for header of output msg */
    char message_buf[BUFNO];     /* fill in text for output msg           */
    long size;                          /* size of struct from fsysc            */
    struct sold_cntl *ctl_ptr;          /* Pointer to SOLD control structure    */
    int istr;                           /* Current I-stream                     */
    unsigned long total_cases;          /* Total test cases run                 */
    unsigned int  total_drivers;        /* Total drivers running                */

    char   sold_token[9];
    THIS_SS_SOLD_CNTL_TOKEN(sold_token);                                        //F11565
                                                                                //F11565
    /* Retrieve the control struct */
    ctl_ptr = tpf_fsysc(sold_token, &size);                                     //C64JK

    /* If this struct does not exist, the driver isn't running. Post error and exit */
    if (ctl_ptr == NULL) {                                                      //C64CF
        wtopc_insert_header(header_buffer, "SOLD", 7, 'I', WTOPC_SYS_TIME);     //C64CF
        wtopc("SOLD IS NOT RUNNING", WTOPC_EBROUT,                              //C64CF
                   WTOPC_NO_CHAIN, header_buffer);                              //C64CF
        exit(-1);                                                               //C64JK
    }                                                                           //C64CF


   /* Print the table header */
   wtopc_insert_header(header_buffer, "SOLD", 2, 'I', WTOPC_SYS_TIME);          //C64CF
   wtopc("SOLD STATUS", WTOPC_EBROUT, WTOPC_CHAIN, header_buffer);              //C64CF
   wtopc("           NMBR OF         TOTAL TEST",                               //C64CF
        WTOPC_EBROUT, WTOPC_CHAIN, WTOPC_NO_HEADER);                            //C64CF
   wtopc("    I-S    DRIVERS          CASES RUN",                               //C64CF
        WTOPC_EBROUT, WTOPC_CHAIN, WTOPC_NO_HEADER);                            //C64CF

   /* Print status of each I-stream that executed tests */
   istr  = 0;                                                                   //C64CF
   total_cases = 0;                                                             //C64CF
   total_drivers = 0;                                                           //C64CF
   while (istr < SAISTRM) {                                                     //C64CF
     if (ctl_ptr->IS_stat[istr].drvr_cnt > 0)  {                                //C64CF
         sprintf(message_buf, "    %3d    %7d    %15ld %s",                     //C64CF
            istr+1, ctl_ptr->IS_stat[istr].drvr_cnt,                            //C64CF
            ctl_ptr->IS_stat[istr].case_cnt,
            ctl_ptr->IS_stat[istr].status == STOP ? "STOPPING" : "");           //C64CF
         wtopc(message_buf, WTOPC_EBROUT, WTOPC_CHAIN, WTOPC_NO_HEADER);        //C64CF
         total_cases   += ctl_ptr->IS_stat[istr].case_cnt;                      //C64CF
         total_drivers += ctl_ptr->IS_stat[istr].drvr_cnt;                      //C64CF
      }                                                                         //C64CF
      istr++;                                                                   //C64CF
   }                                                                            //C64CF
   sprintf(message_buf, "\nTOTALS:    %7d    %15ld", total_drivers, total_cases);
   wtopc(message_buf, WTOPC_EBROUT, WTOPC_CHAIN, WTOPC_NO_HEADER);              //C64CF
   wtopc("END OF DISPLAY", WTOPC_EBROUT, WTOPC_CHAIN_END, WTOPC_NO_HEADER);     //C64CF

   return;                                                                      //C64CF
}                                                                               //C64CF

void show_prog_version(char* name, unsigned int patacnum)                       //f12305
{
   char msg[60];                                                                //f5555
   if (memcmp(name+4, "  ", 2) == 0)                                            //f12305
      sprintf(msg, "Executing program %.4s, ACN= %d", name, patacnum);          //f12305
   else                                                                         //f12305
      sprintf(msg, "Executing program %s, ACN= %d", name, patacnum);            //f12305
   wtopc(msg, WTOPC_PRC, WTOPC_NO_CHAIN, WTOPC_NO_HEADER);                      //f5555
   return;
}
